@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,7 +27,8 @@ class CControllerWidgetTrigOverView extends CControllerWidget {
 		$this->setType(WIDGET_TRIG_OVER);
 		$this->setValidationRules([
 			'name' => 'string',
-			'fields' => 'json'
+			'fields' => 'json',
+			'initial_load' => 'in 0,1'
 		]);
 	}
 
@@ -35,7 +36,8 @@ class CControllerWidgetTrigOverView extends CControllerWidget {
 		$fields = $this->getForm()->getFieldsData();
 
 		$data = [
-			'name' => $this->getInput('name', $this->getDefaultHeader()),
+			'name' => $this->getInput('name', $this->getDefaultName()),
+			'initial_load' => (bool) $this->getInput('initial_load', 0),
 			'style' => $fields['style'],
 			'user' => [
 				'debug_mode' => $this->getDebugMode()
@@ -52,11 +54,18 @@ class CControllerWidgetTrigOverView extends CControllerWidget {
 
 		$problem_options = [
 			'show_suppressed' => $fields['show_suppressed'],
-			'show_recent' => ($fields['show'] == TRIGGERS_OPTION_RECENT_PROBLEM) ? true : null
+			'show_recent' => ($fields['show'] == TRIGGERS_OPTION_RECENT_PROBLEM) ? true : null,
+			'tags' => (array_key_exists('tags', $fields) && $fields['tags']) ? $fields['tags'] : null,
+			'evaltype' => array_key_exists('evaltype', $fields) ? $fields['evaltype'] : TAG_EVAL_TYPE_AND_OR
 		];
 
-		list($data['hosts'], $data['triggers']) = getTriggersOverviewData(getSubGroups($fields['groupids']),
-			$fields['application'], [], $trigger_options, $problem_options
+		$host_options = [
+			'hostids' => $fields['hostids'] ? $fields['hostids'] : null
+		];
+
+		[$data['db_hosts'], $data['db_triggers'], $data['dependencies'], $data['triggers_by_name'],
+			$data['hosts_by_name'], $data['exceeded_limit']
+		] = getTriggersOverviewData(getSubGroups($fields['groupids']), $host_options, $trigger_options, $problem_options
 		);
 
 		$this->setResponse(new CControllerResponseData($data));
