@@ -59,7 +59,6 @@ function condition_type2str($type) {
 		CONDITION_TYPE_DUPTIME => _('Uptime/Downtime'),
 		CONDITION_TYPE_DVALUE => _('Received value'),
 		CONDITION_TYPE_EVENT_ACKNOWLEDGED => _('Event acknowledged'),
-		CONDITION_TYPE_APPLICATION => _('Application'),
 		CONDITION_TYPE_PROXY => _('Proxy'),
 		CONDITION_TYPE_EVENT_TYPE => _('Event type'),
 		CONDITION_TYPE_HOST_METADATA => _('Host metadata'),
@@ -145,7 +144,6 @@ function actionConditionValueToString(array $actions) {
 				case CONDITION_TYPE_DSERVICE_PORT:
 				case CONDITION_TYPE_DUPTIME:
 				case CONDITION_TYPE_DVALUE:
-				case CONDITION_TYPE_APPLICATION:
 				case CONDITION_TYPE_EVENT_TAG:
 				case CONDITION_TYPE_EVENT_TAG_VALUE:
 					$result[$i][$j] = $condition['value'];
@@ -371,6 +369,7 @@ function getActionOperationDescriptions(array $actions, $type) {
 	$hostids = [];
 	$groupids = [];
 	$templateids = [];
+	$scriptids = [];
 
 	foreach ($actions as $i => $action) {
 		$result[$i] = [];
@@ -414,6 +413,8 @@ function getActionOperationDescriptions(array $actions, $type) {
 								$groupids[$host_group['groupid']] = true;
 							}
 						}
+
+						$scriptids[$operation['opcommand']['scriptid']] = true;
 						break;
 
 					case OPERATION_TYPE_GROUP_ADD:
@@ -475,6 +476,8 @@ function getActionOperationDescriptions(array $actions, $type) {
 								$groupids[$host_group['groupid']] = true;
 							}
 						}
+
+						$scriptids[$operation['opcommand']['scriptid']] = true;
 						break;
 				}
 			}
@@ -487,6 +490,7 @@ function getActionOperationDescriptions(array $actions, $type) {
 	$hosts = [];
 	$host_groups = [];
 	$templates = [];
+	$scripts = [];
 
 	if ($media_typeids) {
 		$media_types = API::Mediatype()->get([
@@ -500,7 +504,7 @@ function getActionOperationDescriptions(array $actions, $type) {
 		$fullnames = [];
 
 		$users = API::User()->get([
-			'output' => ['userid', 'alias', 'name', 'surname'],
+			'output' => ['userid', 'username', 'name', 'surname'],
 			'userids' => $userids
 		]);
 
@@ -541,6 +545,15 @@ function getActionOperationDescriptions(array $actions, $type) {
 		]);
 	}
 
+	if ($scriptids) {
+		$scripts = API::Script()->get([
+			'output' => ['name'],
+			'scriptids' => array_keys($scriptids),
+			'filter' => ['scope' => ZBX_SCRIPT_SCOPE_ACTION],
+			'preservekeys' => true
+		]);
+	}
+
 	// Format the HTML output.
 	foreach ($actions as $i => $action) {
 		if ($type == ACTION_OPERATION) {
@@ -592,13 +605,15 @@ function getActionOperationDescriptions(array $actions, $type) {
 						break;
 
 					case OPERATION_TYPE_COMMAND:
+						$scriptid = $operation['opcommand']['scriptid'];
+
 						if (array_key_exists('opcommand_hst', $operation) && $operation['opcommand_hst']) {
 							$host_list = [];
 
 							foreach ($operation['opcommand_hst'] as $host) {
 								if ($host['hostid'] == 0) {
 									$result[$i][$j][] = [
-										bold(_('Run remote commands on current host')),
+										bold(_s('Run script "%1$s" on current host', $scripts[$scriptid]['name'])),
 										BR()
 									];
 								}
@@ -610,7 +625,9 @@ function getActionOperationDescriptions(array $actions, $type) {
 							if ($host_list) {
 								order_result($host_list);
 
-								$result[$i][$j][] = bold(_('Run remote commands on hosts').': ');
+								$result[$i][$j][] = bold(
+									_s('Run script "%1$s" on hosts', $scripts[$scriptid]['name']).': '
+								);
 								$result[$i][$j][] = [implode(', ', $host_list), BR()];
 							}
 						}
@@ -626,7 +643,9 @@ function getActionOperationDescriptions(array $actions, $type) {
 
 							order_result($host_group_list);
 
-							$result[$i][$j][] = bold(_('Run remote commands on host groups').': ');
+							$result[$i][$j][] = bold(
+								_s('Run script "%1$s" on host groups', $scripts[$scriptid]['name']).': '
+							);
 							$result[$i][$j][] = [implode(', ', $host_group_list), BR()];
 						}
 						break;
@@ -755,13 +774,15 @@ function getActionOperationDescriptions(array $actions, $type) {
 						break;
 
 					case OPERATION_TYPE_COMMAND:
+						$scriptid = $operation['opcommand']['scriptid'];
+
 						if (array_key_exists('opcommand_hst', $operation) && $operation['opcommand_hst']) {
 							$host_list = [];
 
 							foreach ($operation['opcommand_hst'] as $host) {
 								if ($host['hostid'] == 0) {
 									$result[$i][$j][] = [
-										bold(_('Run remote commands on current host')),
+										bold(_s('Run script "%1$s" on current host', $scripts[$scriptid]['name'])),
 										BR()
 									];
 								}
@@ -773,7 +794,9 @@ function getActionOperationDescriptions(array $actions, $type) {
 							if ($host_list) {
 								order_result($host_list);
 
-								$result[$i][$j][] = bold(_('Run remote commands on hosts').': ');
+								$result[$i][$j][] = bold(
+									_s('Run script "%1$s" on hosts', $scripts[$scriptid]['name']).': '
+								);
 								$result[$i][$j][] = [implode(', ', $host_list), BR()];
 							}
 						}
@@ -789,7 +812,9 @@ function getActionOperationDescriptions(array $actions, $type) {
 
 							order_result($host_group_list);
 
-							$result[$i][$j][] = bold(_('Run remote commands on host groups').': ');
+							$result[$i][$j][] = bold(
+								_s('Run script "%1$s" on host groups', $scripts[$scriptid]['name']).': '
+							);
 							$result[$i][$j][] = [implode(', ', $host_group_list), BR()];
 						}
 						break;
@@ -806,102 +831,6 @@ function getActionOperationDescriptions(array $actions, $type) {
 	return $result;
 }
 
-/**
- * Gathers action operation script details and returns the HTML items representing action operation with hint.
- *
- * @param array  $operations                      Array of action operations.
- * @param string $operation['operationtype']      Action operation type.
- *                                                Possible values: OPERATION_TYPE_MESSAGE, OPERATION_TYPE_COMMAND,
- *                                                OPERATION_TYPE_ACK_MESSAGE and OPERATION_TYPE_RECOVERY_MESSAGE
- * @param string $operation['opcommand']['type']  Action operation command type.
- *                                                Possible values: ZBX_SCRIPT_TYPE_IPMI, ZBX_SCRIPT_TYPE_SSH,
- *                                                ZBX_SCRIPT_TYPE_TELNET, ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT
- *                                                and ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT
- *
- * @return array  Returns an array of action operation hints.
- */
-function getActionOperationHints(array $operations) {
-	$result = [];
-	$scriptids = [];
-	$scripts = [];
-
-	foreach ($operations as $operation) {
-		if ($operation['operationtype'] == OPERATION_TYPE_COMMAND
-				&& $operation['opcommand']['type'] == ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT) {
-			$scriptids[$operation['opcommand']['scriptid']] = true;
-		}
-	}
-
-	if ($scriptids) {
-		$scripts = API::Script()->get([
-			'output' => ['name'],
-			'scriptids' => array_keys($scriptids),
-			'preservekeys' => true
-		]);
-	}
-
-	foreach ($operations as $key => $operation) {
-		$result[$key] = [];
-
-		if ($operation['operationtype'] == OPERATION_TYPE_COMMAND) {
-			switch ($operation['opcommand']['type']) {
-				case ZBX_SCRIPT_TYPE_IPMI:
-					$result[$key][] = [bold(_('Run IPMI command').': '), BR(),
-						italic(zbx_nl2br($operation['opcommand']['command']))
-					];
-					break;
-
-				case ZBX_SCRIPT_TYPE_SSH:
-					$result[$key][] = [bold(_('Run SSH commands').': '), BR(),
-						italic(zbx_nl2br($operation['opcommand']['command']))
-					];
-					break;
-
-				case ZBX_SCRIPT_TYPE_TELNET:
-					$result[$key][] = [bold(_('Run TELNET commands').': '), BR(),
-						italic(zbx_nl2br($operation['opcommand']['command']))
-					];
-					break;
-
-				case ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT:
-					if ($operation['opcommand']['execute_on'] == ZBX_SCRIPT_EXECUTE_ON_AGENT) {
-						$result[$key][] = [bold(_s('Run custom commands on %1$s', _('Zabbix agent')).': '),
-							BR(), italic(zbx_nl2br($operation['opcommand']['command']))
-						];
-					}
-					elseif ($operation['opcommand']['execute_on'] == ZBX_SCRIPT_EXECUTE_ON_PROXY) {
-						$result[$key][] = [bold(_s('Run custom commands on %1$s', _('Zabbix server (proxy)')).': '),
-							BR(), italic(zbx_nl2br($operation['opcommand']['command']))
-						];
-					}
-					else {
-						$result[$key][] = [bold(_s('Run custom commands on %1$s', _('Zabbix server')).': '),
-							BR(), italic(zbx_nl2br($operation['opcommand']['command']))
-						];
-					}
-					break;
-
-				case ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT:
-					$scriptId = $operation['opcommand']['scriptid'];
-
-					if (isset($scripts[$scriptId])) {
-						$result[$key][] = [bold(_('Run global script').': '),
-							italic($scripts[$scriptId]['name'])
-						];
-					}
-					break;
-
-				default:
-					$result[$key][] = [bold(_('Run commands').': '), BR(),
-						italic(zbx_nl2br($operation['opcommand']['command']))
-					];
-			}
-		}
-	}
-
-	return $result;
-}
-
 /**
  * Return an array of action conditions supported by the given event source.
  *
@@ -914,7 +843,6 @@ function get_conditions_by_eventsource($eventsource) {
 		CONDITION_TYPE_TRIGGER_NAME,
 		CONDITION_TYPE_TRIGGER,
 		CONDITION_TYPE_TRIGGER_SEVERITY,
-		CONDITION_TYPE_APPLICATION,
 		CONDITION_TYPE_HOST,
 		CONDITION_TYPE_HOST_GROUP,
 		CONDITION_TYPE_SUPPRESSED,
@@ -941,10 +869,11 @@ function get_conditions_by_eventsource($eventsource) {
 		CONDITION_TYPE_PROXY
 	];
 	$conditions[EVENT_SOURCE_INTERNAL] = [
-		CONDITION_TYPE_APPLICATION,
 		CONDITION_TYPE_EVENT_TYPE,
 		CONDITION_TYPE_HOST,
 		CONDITION_TYPE_HOST_GROUP,
+		CONDITION_TYPE_EVENT_TAG,
+		CONDITION_TYPE_EVENT_TAG_VALUE,
 		CONDITION_TYPE_TEMPLATE
 	];
 
@@ -1187,11 +1116,6 @@ function get_operators_by_conditiontype($conditiontype) {
 	$operators[CONDITION_TYPE_EVENT_ACKNOWLEDGED] = [
 		CONDITION_OPERATOR_EQUAL
 	];
-	$operators[CONDITION_TYPE_APPLICATION] = [
-		CONDITION_OPERATOR_EQUAL,
-		CONDITION_OPERATOR_LIKE,
-		CONDITION_OPERATOR_NOT_LIKE
-	];
 	$operators[CONDITION_TYPE_HOST_NAME] = [
 		CONDITION_OPERATOR_LIKE,
 		CONDITION_OPERATOR_NOT_LIKE,
@@ -1300,23 +1224,21 @@ function eventType($type = null) {
  *
  * @param array $events    Array with event objects with acknowledges.
  * @param array $triggers  Array of triggers.
- * @param array $r_events  Array with related recovery event data (optional).
  *
  * @return array
  */
-function getEventsActionsIconsData(array $events, array $triggers, array $r_events = []) {
+function getEventsActionsIconsData(array $events, array $triggers) {
 	$messages = getEventsMessages($events);
 	$severities = getEventsSeverityChanges($events, $triggers);
-	$actions = getEventsActions($events, $r_events);
+	$actions = getEventsAlertsOverview($events);
 
 	return [
 		'data' => [
 			'messages' => $messages['data'],
 			'severities' => $severities['data'],
-			'actions' => $actions['data']
+			'actions' => $actions
 		],
-		'mediatypeids' => $actions['mediatypeids'],
-		'userids' => $messages['userids'] + $severities['userids'] + $actions['userids']
+		'userids' => $messages['userids'] + $severities['userids']
 	];
 }
 
@@ -1343,7 +1265,6 @@ function getEventsMessages(array $events) {
 
 		foreach ($event['acknowledges'] as $ack) {
 			if (($ack['action'] & ZBX_PROBLEM_UPDATE_MESSAGE) == ZBX_PROBLEM_UPDATE_MESSAGE) {
-				// Alias is mandatory for each user, so if alias is not returned, we don't have rights for this user.
 				$event_messages[] = [
 					'message' => $ack['message'],
 					'userid' => $ack['userid'],
@@ -1424,21 +1345,18 @@ function getEventsSeverityChanges(array $events, array $triggers) {
 }
 
 /**
- * Get data, required to create actions icon with popup with event actions.
+ * Get data, required to create actions icon.
  *
  * @param array  $events                 Array with event objects with acknowledges.
  * @param string $events[]['eventid']    Problem event ID.
  * @param string $events[]['r_eventid']  OK event ID.
- * @param array  $r_events               Array with related recovery event data (optional).
  *
- * @return array
+ * @return array  List indexed by eventid containing overview on event alerts.
  */
-function getEventsActions(array $events, array $r_events = []) {
+function getEventsAlertsOverview(array $events): array {
 	$alert_eventids = [];
-	$r_eventids = [];
-	$userids = [];
-	$mediatypeids = [];
 	$actions = [];
+	$event_alert_state = [];
 
 	foreach ($events as $event) {
 		// Get alerts for event.
@@ -1447,43 +1365,67 @@ function getEventsActions(array $events, array $r_events = []) {
 		// Get alerts for related recovery events.
 		if ($event['r_eventid'] != 0) {
 			$alert_eventids[$event['r_eventid']] = true;
-			$r_eventids[$event['r_eventid']] = true;
 		}
 	}
 
-	if ($r_eventids && !$r_events) {
-		$r_events = API::Event()->get([
-			'output' => ['clock'],
-			'eventids' => array_keys($r_eventids),
-			'preservekeys' => true
-		]);
-	}
+	if ($alert_eventids) {
+		$event_alert_state = array_combine(array_keys($alert_eventids), array_fill(0, count($alert_eventids), [
+			'failed_cnt' => 0,
+			'in_progress_cnt' => 0,
+			'total_cnt' => 0
+		]));
 
-	$alerts = $alert_eventids
-		? API::Alert()->get([
-			'output' => ['alerttype', 'clock', 'error', 'eventid', 'mediatypeid', 'retries', 'status', 'userid'],
+		$alerts = API::Alert()->get([
+			'groupCount' => true,
+			'countOutput' => true,
+			'filter' => ['status' => ALERT_STATUS_FAILED],
 			'eventids' => array_keys($alert_eventids)
-		])
-		: [];
+		]);
+
+		foreach ($alerts as $alert) {
+			$event_alert_state[$alert['eventid']]['failed_cnt'] = (int) $alert['rowscount'];
+		}
+
+		$alerts = API::Alert()->get([
+			'groupCount' => true,
+			'countOutput' => true,
+			'filter' => ['status' => [ALERT_STATUS_NEW, ALERT_STATUS_NOT_SENT]],
+			'eventids' => array_keys($alert_eventids)
+		]);
+
+		foreach ($alerts as $alert) {
+			$event_alert_state[$alert['eventid']]['in_progress_cnt'] = (int) $alert['rowscount'];
+		}
+
+		$alerts = API::Alert()->get([
+			'groupCount' => true,
+			'countOutput' => true,
+			'eventids' => array_keys($alert_eventids)
+		]);
+
+		foreach ($alerts as $alert) {
+			$event_alert_state[$alert['eventid']]['total_cnt'] = (int) $alert['rowscount'];
+		}
+	}
 
 	// Create array of actions for each event.
 	foreach ($events as $event) {
-		$event_actions = getSingleEventActions($event, $r_events, $alerts);
+		$event_actions = $event_alert_state[$event['eventid']];
+		if ($event['r_eventid']) {
+			$r_event_actions = $event_alert_state[$event['r_eventid']];
+			$event_actions['failed_cnt'] += $r_event_actions['failed_cnt'];
+			$event_actions['total_cnt'] += $r_event_actions['total_cnt'];
+			$event_actions['in_progress_cnt'] += $r_event_actions['in_progress_cnt'];
+		}
+
 		$actions[$event['eventid']] = [
-			'actions' => $event_actions['actions'],
-			'count' => $event_actions['count'],
-			'has_uncomplete_action' => $event_actions['has_uncomplete_action'],
-			'has_failed_action' => $event_actions['has_failed_action']
+			'count' => $event_actions['total_cnt'] + count($event['acknowledges']),
+			'has_uncomplete_action' => (bool) $event_actions['in_progress_cnt'],
+			'has_failed_action' => (bool) $event_actions['failed_cnt']
 		];
-		$mediatypeids += $event_actions['mediatypeids'];
-		$userids += $event_actions['userids'];
 	}
 
-	return [
-		'data' => $actions,
-		'mediatypeids' => $mediatypeids,
-		'userids' => $userids
-	];
+	return $actions;
 }
 
 /**
@@ -1511,12 +1453,15 @@ function getEventDetailsActions(array $event) {
 		]);
 	}
 
+	$search_limit = CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT);
+
 	// Get automatic actions (alerts).
 	$alerts = API::Alert()->get([
 		'output' => ['alerttype', 'clock', 'error', 'eventid', 'esc_step', 'mediatypeid', 'message', 'retries',
 			'sendto', 'status', 'subject', 'userid', 'p_eventid', 'acknowledgeid'
 		],
-		'eventids' => $alert_eventids
+		'eventids' => $alert_eventids,
+		'config' => $search_limit
 	]);
 
 	$actions = getSingleEventActions($event, $r_events, $alerts);
@@ -1524,7 +1469,8 @@ function getEventDetailsActions(array $event) {
 	return [
 		'actions' => $actions['actions'],
 		'mediatypeids' => $actions['mediatypeids'],
-		'userids' => $actions['userids']
+		'userids' => $actions['userids'],
+		'count' => $actions['count']
 	];
 }
 
@@ -1659,15 +1605,14 @@ function getEventUpdates(array $event) {
  * @param array  $actions['messages']    Messages icon data.
  * @param array  $actions['severities']  Severity change icon data.
  * @param array  $actions['actions']     Actions icon data.
- * @param array  $mediatypes             Mediatypes with maxattempts value and name.
- * @param array  $users                  User name, surname and alias.
+ * @param array  $users                  User name, surname and username.
  *
  * @return CCol|string
  */
-function makeEventActionsIcons($eventid, $actions, $mediatypes, $users) {
+function makeEventActionsIcons($eventid, $actions, $users) {
 	$messages_icon = makeEventMessagesIcon($actions['messages'][$eventid], $users);
 	$severities_icon = makeEventSeverityChangesIcon($actions['severities'][$eventid], $users);
-	$actions_icon = makeEventActionsIcon($actions['actions'][$eventid], $users, $mediatypes);
+	$actions_icon = makeEventActionsIcon($actions['actions'][$eventid], $eventid);
 
 	$action_icons = [];
 	if ($messages_icon !== null) {
@@ -1690,11 +1635,11 @@ function makeEventActionsIcons($eventid, $actions, $mediatypes, $users) {
  * @param array  $data['messages']               Array of messages.
  * @param string $data['messages'][]['message']  Message text.
  * @param string $data['messages'][]['clock']    Message creation time.
- * @param array  $users                          User name, surname and alias.
+ * @param array  $users                          User name, surname and username.
  *
- * @return CSpan|null
+ * @return CButton|null
  */
-function makeEventMessagesIcon(array $data, array $users) {
+function makeEventMessagesIcon(array $data, array $users): ?CButton {
 	$total = $data['count'];
 
 	$table = (new CTableInfo())->setHeader([_('Time'), _('User'), _('Message')]);
@@ -1715,6 +1660,7 @@ function makeEventMessagesIcon(array $data, array $users) {
 	return $total
 		? makeActionIcon([
 			'icon' => ZBX_STYLE_ACTION_ICON_MSGS,
+			'button' => true,
 			'hint' => [
 				$table,
 				($total > ZBX_WIDGET_ROWS)
@@ -1743,11 +1689,11 @@ function makeEventMessagesIcon(array $data, array $users) {
  * @param string $data['original_severity']             Severity before change.
  * @param string $data['current_severity']              Current severity.
  * @param int    $data['count']                         Total number of severity changes.
- * @param array  $users                                 User name, surname and alias.
+ * @param array  $users                                 User name, surname and username.
  *
- * @return CSpan|null
+ * @return CButton|null
  */
-function makeEventSeverityChangesIcon(array $data, array $users) {
+function makeEventSeverityChangesIcon(array $data, array $users): ?CButton {
 	$total = $data['count'];
 
 	$table = (new CTableInfo())->setHeader([_('Time'), _('User'), _('Severity changes')]);
@@ -1785,6 +1731,7 @@ function makeEventSeverityChangesIcon(array $data, array $users) {
 
 	return $total
 		? makeActionIcon([
+			'button' => true,
 			'icon' => $icon_style,
 			'hint' => [
 				$table,
@@ -1803,38 +1750,30 @@ function makeEventSeverityChangesIcon(array $data, array $users) {
 }
 
 /**
- * Create icon with hintbox for event actions.
+ * @param array  $actions                      Array with all actions sorted by clock.
+ * @param int    $actions[]['action_type']     Type of action table entry (ZBX_EVENT_HISTORY_*).
+ * @param string $actions[]['clock']           Time, when action was performed.
+ * @param string $actions[]['message']         Message sent by alert, or written by manual update, or remote command text.
+ * @param string $actions[]['action']          Flag with problem update operation performed (only for ZBX_EVENT_HISTORY_MANUAL_UPDATE).
+ * @param string $actions[]['alerttype']       Type of alert (only for ZBX_EVENT_HISTORY_ALERT).
+ * @param string $actions[]['mediatypeid']     Id for mediatype, where alert message was sent (only for ZBX_EVENT_HISTORY_ALERT).
+ * @param string $actions[]['error']           Error message in case of failed alert (only for ZBX_EVENT_HISTORY_ALERT).
+ * @param array  $users                        User name, surname and username.
+ * @param array  $mediatypes                   Mediatypes with maxattempts value and name.
+ * @param string $mediatypes[]['name']         Mediatype name.
+ * @param string $mediatypes[]['maxattempts']  Maximum attempts for this mediatype.
  *
- * @param array  $data
- * @param array  $data['actions']                   Array with all actions sorted by clock.
- * @param int    $data['actions'][]['action_type']  Type of action table entry (ZBX_EVENT_HISTORY_*).
- * @param string $data['actions'][]['clock']        Time, when action was performed.
- * @param string $data['actions'][]['message']      Message sent by alert, or written by manual update, or remote command text.
- * @param string $data['actions'][]['action']       Flag with problem update operation performed (only for ZBX_EVENT_HISTORY_MANUAL_UPDATE).
- * @param string $data['actions'][]['alerttype']    Type of alert (only for ZBX_EVENT_HISTORY_ALERT).
- * @param string $data['actions'][]['mediatypeid']  Id for mediatype, where alert message was sent (only for ZBX_EVENT_HISTORY_ALERT).
- * @param string $data['actions'][]['error']        Error message in case of failed alert (only for ZBX_EVENT_HISTORY_ALERT).
- * @param int    $data['count']                     Number of actions.
- * @param bool   $data['has_uncomplete_action']     Does the event have at least one uncompleted alert action.
- * @param bool   $data['has_failed_action']         Does the event have at least one failed alert action.
- * @param array  $users                             User name, surname and alias.
- * @param array  $mediatypes                        Mediatypes with maxattempts value and name.
- * @param string $mediatypes[]['name']              Mediatype name.
- *
- * @return CSpan|null
+ * @return CTableInfo
  */
-function makeEventActionsIcon(array $data, array $users, array $mediatypes) {
-	// Number of meaningful actions.
-	$total = $data['count'];
-	// Number of all action entries.
-	$action_count = count($data['actions']);
+function makeEventActionsTable(array $actions, array $users, array $mediatypes): CTableInfo {
+	$action_count = count($actions);
 
 	$table = (new CTableInfo())->setHeader([
 		_('Time'), _('User/Recipient'), _('Action'), _('Message/Command'), _('Status'), _('Info')
 	]);
 
 	for ($i = 0; $i < $action_count && $i < ZBX_WIDGET_ROWS; $i++) {
-		$action = $data['actions'][$i];
+		$action = $actions[$i];
 
 		$message = '';
 		if ($action['action_type'] == ZBX_EVENT_HISTORY_MANUAL_UPDATE
@@ -1862,6 +1801,24 @@ function makeEventActionsIcon(array $data, array $users, array $mediatypes) {
 		]);
 	}
 
+	return $table;
+}
+
+/**
+ * Create icon with hintbox for event actions.
+ *
+ * @param array  $data
+ * @param int    $data['count']                     Number of actions.
+ * @param bool   $data['has_uncomplete_action']     Does the event have at least one uncompleted alert action.
+ * @param bool   $data['has_failed_action']         Does the event have at least one failed alert action.
+ * @param string $eventid
+ *
+ * @return CButton|null
+ */
+function makeEventActionsIcon(array $data, $eventid): ?CButton {
+	// Number of meaningful actions.
+	$total = $data['count'];
+
 	// select icon
 	if ($data['has_failed_action']) {
 		$icon_style = ZBX_STYLE_ACTIONS_NUM_RED;
@@ -1876,19 +1833,12 @@ function makeEventActionsIcon(array $data, array $users, array $mediatypes) {
 	return $total
 		? makeActionIcon([
 			'icon' => $icon_style,
-			'hint' => [
-				$table,
-				($total > ZBX_WIDGET_ROWS)
-					? (new CDiv(
-						(new CDiv(
-							(new CDiv(_s('Displaying %1$s of %2$s found', ZBX_WIDGET_ROWS, $total)))
-								->addClass(ZBX_STYLE_TABLE_STATS)
-						))->addClass(ZBX_STYLE_PAGING_BTN_CONTAINER)
-					))->addClass(ZBX_STYLE_TABLE_PAGING)
-					: null
-			],
+			'button' => true,
 			'num' => $total,
 			'aria-label' => _xn('%1$s action', '%1$s actions', $total, 'screen reader', $total)
+		])->setAjaxHint([
+			'type' => 'eventactions',
+			'data' => ['eventid' => $eventid]
 		])
 		: null;
 }
@@ -1906,7 +1856,7 @@ function makeEventActionsIcon(array $data, array $users, array $mediatypes) {
  * @param string $data['actions'][]['subject']        Message alert subject (only for ZBX_EVENT_HISTORY_ALERT).
  * @param string $data['actions'][]['p_eventid']      Problem eventid that was reason for alert (only for ZBX_EVENT_HISTORY_ALERT).
  * @param string $data['actions'][]['acknowledgeid']  Problem update action that was reason for alert (only for ZBX_EVENT_HISTORY_ALERT).
- * @param array  $users                               User name, surname and alias.
+ * @param array  $users                               User name, surname and username.
  * @param array  $mediatypes                          Mediatypes with maxattempts value.
  *
  * @return CTableInfo
@@ -1934,7 +1884,11 @@ function makeEventDetailsActionsTable(array $data, array $users, array $mediatyp
 		}
 		elseif (($action['action_type'] == ZBX_EVENT_HISTORY_ALERT && $action['alerttype'] == ALERT_TYPE_COMMAND)
 				|| $action['action_type'] == ZBX_EVENT_HISTORY_MANUAL_UPDATE) {
-			$message = zbx_nl2br($action['message']);
+			$message = [
+				bold(_('Command').':'),
+				BR(),
+				zbx_nl2br($action['message'])
+			];
 		}
 
 		$table->addRow([
@@ -1957,7 +1911,7 @@ function makeEventDetailsActionsTable(array $data, array $users, array $mediatyp
  * @param array  $actions                   Array with all actions sorted by clock.
  * @param string $actions[]['clock']        Time, when action was performed.
  * @param string $actions[]['message']      Message sent by alert, or written by manual update, or remote command text.
- * @param array  $users                     User name, surname and alias.
+ * @param array  $users                     User name, surname and username.
  *
  * @return CTable
  */
@@ -1988,7 +1942,7 @@ function makeEventHistoryTable(array $actions, array $users) {
  * @param int    $action['action_type']  Type of event table action (ZBX_EVENT_HISTORY_*).
  * @param string $action['alerttype']    Type of alert.
  * @param string $action['userid']       ID of message author, or alert receiver.
- * @param array  $users                  Array with user data - alias, name, surname.
+ * @param array  $users                  Array with user data - username, name, surname.
  *
  * @return string
  */
@@ -2012,7 +1966,7 @@ function makeActionTableUser(array $action, array $users) {
  * @param string $action['alerttype']    Type of alert.
  * @param array  $action['userid']       ID of message author, or alert receiver.
  * @param array  $action['sendto']       Receiver media address for automatic action.
- * @param array  $users                  Array with user data - alias, name, surname.
+ * @param array  $users                  Array with user data - username, name, surname.
  *
  * @return string
  */
@@ -2083,7 +2037,7 @@ function makeActionTableIcon(array $action) {
 				$new_severity_name = getSeverityName($action['new_severity']);
 				$hint = $old_severity_name.'&nbsp;&rArr;&nbsp;'.$new_severity_name;
 
-				$action_icons[] = makeActionIcon(['icon' => $action_type, 'hint' => $hint]);
+				$action_icons[] = makeActionIcon(['button' => true, 'icon' => $action_type, 'hint' => $hint]);
 			}
 
 			return (new CCol($action_icons))->addClass(ZBX_STYLE_NOWRAP);
