@@ -90,6 +90,7 @@ class CAlert extends CApiService {
 			'selectUsers'				=> null,
 			'selectHosts'				=> null,
 			'countOutput'				=> false,
+			'groupCount'				=> false,
 			'preservekeys'				=> false,
 			'editable'					=> false,
 			'sortfield'					=> '',
@@ -264,6 +265,10 @@ class CAlert extends CApiService {
 			zbx_value2array($options['eventids']);
 
 			$sqlParts['where'][] = dbConditionInt('a.eventid', $options['eventids']);
+
+			if ($options['groupCount']) {
+				$sqlParts['group']['a'] = 'a.eventid';
+			}
 		}
 
 		// actionids
@@ -330,7 +335,12 @@ class CAlert extends CApiService {
 		$dbRes = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($alert = DBfetch($dbRes)) {
 			if ($options['countOutput']) {
-				$result = $alert['rowscount'];
+				if ($options['groupCount']) {
+					$result[] = $alert;
+				}
+				else {
+					$result = $alert['rowscount'];
+				}
 			}
 			else {
 				$result[$alert['alertid']] = $alert;
