@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,13 @@
  */
 class CCorrelation extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_SUPER_ADMIN]
+	];
+
 	protected $tableName = 'correlation';
 	protected $tableAlias = 'c';
 	protected $sortColumns = ['correlationid', 'name', 'status'];
@@ -49,13 +56,13 @@ class CCorrelation extends CApiService {
 	 *
 	 * @param array $options
 	 *
-	 * @return array
+	 * @return array|string
 	 */
 	public function get($options = []) {
 		$options = zbx_array_merge($this->getOptions, $options);
 
 		if ($options['editable'] && self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			return ($options['countOutput'] && !$options['groupCount']) ? 0 : [];
+			return ($options['countOutput'] && !$options['groupCount']) ? '0' : [];
 		}
 
 		$res = DBselect($this->createSelectQuery($this->tableName(), $options), $options['limit']);
@@ -356,7 +363,7 @@ class CCorrelation extends CApiService {
 				// Check if evaltype has changed.
 				if (array_key_exists('evaltype', $correlation['filter'])) {
 					if ($correlation['filter']['evaltype'] != $db_correlation['filter']['evaltype']) {
-						// Clear formula field evaluation method is changed and no longer a custom experssion.
+						// Clear formula field evaluation method if evaltype has changed.
 						$correlation['evaltype'] = $correlation['filter']['evaltype'];
 
 						if ($correlation['evaltype'] != CONDITION_EVAL_TYPE_EXPRESSION) {
@@ -953,7 +960,7 @@ class CCorrelation extends CApiService {
 	 * depend on it. As a result return host group IDs that need to be validated afterwards. Otherwise don't return
 	 * anything, just throw an error.
 	 *
-	 * @param array					$correlation											One correlation contaning the conditions.
+	 * @param array					$correlation											One correlation containing the conditions.
 	 * @param string				$correlation['name']									Correlation name for error messages.
 	 * @param array					$correlation['filter']									Correlation filter array containing	the conditions.
 	 * @param array					$correlation['filter']['conditions']					An array of correlation conditions.
@@ -1144,6 +1151,13 @@ class CCorrelation extends CApiService {
 								_('Incorrect arguments passed to function.')
 							);
 						}
+						elseif (($condition['operator'] == CONDITION_OPERATOR_LIKE
+									|| $condition['operator'] == CONDITION_OPERATOR_NOT_LIKE)
+								&& (!array_key_exists('value', $condition) || $condition['value'] === '')) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Incorrect value for field "%1$s": %2$s.', 'value', _('cannot be empty'))
+							);
+						}
 						elseif (!$filter_condition_tagval_operator_validator->validate($condition['operator'])) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _s(
 								'Incorrect value "%1$s" in field "%2$s" for correlation "%3$s".',
@@ -1184,7 +1198,7 @@ class CCorrelation extends CApiService {
 	/**
 	 * Validate correlation filter "formula" field.
 	 *
-	 * @param array				$correlation						One correlation contaning the filter, formula and name.
+	 * @param array				$correlation						One correlation containing the filter, formula and name.
 	 * @param string			$correlation['name']				Correlation name for error messages.
 	 * @param array				$correlation['filter']				Correlation filter array containing the formula.
 	 * @param string			$correlation['filter']['formula']	User-defined expression to be used for evaluating
@@ -1210,7 +1224,7 @@ class CCorrelation extends CApiService {
 	/**
 	 * Validate correlation condition formula IDs. Check the "formulaid" field and that formula matches the conditions.
 	 *
-	 * @param array				$correlation										One correlation contaning array of
+	 * @param array				$correlation										One correlation containing array of
 	 *																				conditions and name.
 	 * @param string			$correlation['name']								Correlation name for error messages.
 	 * @param array				$correlation['filter']								Correlation filter array containing
@@ -1266,7 +1280,7 @@ class CCorrelation extends CApiService {
 	 * Validate correlation operations. Check if "operations" is valid, if "type" is valid and there are no duplicate
 	 * operations in correlation.
 	 *
-	 * @param array					$correlation						One correlation contaning array of operations and name.
+	 * @param array					$correlation						One correlation containing array of operations and name.
 	 * @param string				$correlation['name']				Correlation name for error messages.
 	 * @param array					$correlation['operations']			An array of correlation operations.
 	 * @param int					$correlation['operations']['type']	Correlation operation type.
