@@ -46,16 +46,18 @@ abstract class CGraphGeneral extends CApiService {
 	 */
 	public function update(array $graphs) {
 		$graphs = zbx_toArray($graphs);
-		$graphIds = zbx_objectValues($graphs, 'graphid');
+		$graphids = array_column($graphs, 'graphid');
 
 		$graphs = $this->extendObjects($this->tableName(), $graphs,
 			['name', 'graphtype', 'ymin_type', 'ymin_itemid', 'ymax_type', 'ymax_itemid', 'yaxismin', 'yaxismax']
 		);
 
-		$dbGraphs = $this->get([
+		$db_graphs = $this->get([
 			'output' => API_OUTPUT_EXTEND,
-			'selectGraphItems' => API_OUTPUT_EXTEND,
-			'graphids' => $graphIds,
+			'selectGraphItems' => ['gitemid', 'itemid', 'drawtype', 'sortorder', 'color', 'yaxisside', 'calc_fnc',
+				'type'
+			],
+			'graphids' => $graphids,
 			'editable' => true,
 			'preservekeys' => true,
 			'inherited' => false
@@ -65,49 +67,47 @@ abstract class CGraphGeneral extends CApiService {
 			'messageAllowed' => _('Cannot update a discovered graph.')
 		]);
 
+		foreach ($db_graphs as &$db_graph) {
+			$db_graph['gitems'] = array_column($db_graph['gitems'], null, 'gitemid');
+		}
+		unset($db_graph);
+
 		foreach ($graphs as &$graph) {
 			// check permissions
-			if (!isset($dbGraphs[$graph['graphid']])) {
+			if (!array_key_exists($graph['graphid'], $db_graphs)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('No permissions to referred object or it does not exist!'));
 			}
 
 			// cannot update discovered graphs
-			$this->checkPartialValidator($graph, $updateDiscoveredValidator, $dbGraphs[$graph['graphid']]);
+			$this->checkPartialValidator($graph, $updateDiscoveredValidator, $db_graphs[$graph['graphid']]);
 
 			// validate items on set or pass existing items from DB
-			if (isset($graph['gitems'])) {
-				foreach ($graph['gitems'] as $item) {
-					if (isset($item['gitemid']) && !$item['gitemid']) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _('Missing "gitemid" field for item.'));
-					}
-
-					if (isset($item['gitemid']) && $item['gitemid']) {
-						$validGraphItemIds = [];
-
-						foreach ($dbGraphs[$graph['graphid']]['gitems'] as $dbItem) {
-							$validGraphItemIds[$dbItem['gitemid']] = $dbItem['gitemid'];
-						}
-
-						if (!in_array($item['gitemid'], $validGraphItemIds)) {
+			if (array_key_exists('gitems', $graph)) {
+				foreach ($graph['gitems'] as &$gitem) {
+					if (array_key_exists('gitemid', $gitem)) {
+						if (!array_key_exists($gitem['gitemid'], $db_graphs[$graph['graphid']]['gitems'])) {
 							self::exception(ZBX_API_ERROR_PARAMETERS,
 								_('No permissions to referred object or it does not exist!')
 							);
 						}
+
+						$gitem += $db_graphs[$graph['graphid']]['gitems'][$gitem['gitemid']];
 					}
 				}
+				unset($gitem);
 			}
 			else {
-				$graph['gitems'] = $dbGraphs[$graph['graphid']]['gitems'];
+				$graph['gitems'] = $db_graphs[$graph['graphid']]['gitems'];
 			}
 		}
 		unset($graph);
 
-		$this->validateUpdate($graphs, $dbGraphs);
+		$this->validateUpdate($graphs, $db_graphs);
 
-		foreach ($graphs as $graph) {
+		foreach ($graphs as &$graph) {
 			unset($graph['templateid']);
 
-			$graph['gitems'] = isset($graph['gitems']) ? $graph['gitems'] : $dbGraphs[$graph['graphid']]['gitems'];
+			$graph['gitems'] = isset($graph['gitems']) ? $graph['gitems'] : $db_graphs[$graph['graphid']]['gitems'];
 
 			// Y axis min clean unused fields
 			if (isset($graph['ymin_type'])) {
@@ -128,12 +128,13 @@ abstract class CGraphGeneral extends CApiService {
 					$graph['ymax_itemid'] = null;
 				}
 			}
-
-			$this->updateReal($graph, $dbGraphs[$graph['graphid']]);
-			$this->inherit($graph);
 		}
+		unset($graph);
 
-		return ['graphids' => $graphIds];
+		$this->updateReal($graphs);
+		$this->inherit($graphs);
+
+		return ['graphids' => $graphids];
 	}
 
 	/**
@@ -145,7 +146,6 @@ abstract class CGraphGeneral extends CApiService {
 	 */
 	public function create(array $graphs) {
 		$graphs = zbx_toArray($graphs);
-		$graphids = [];
 
 		// set default parameters
 		foreach ($graphs as &$graph) {
@@ -163,102 +163,124 @@ abstract class CGraphGeneral extends CApiService {
 
 		$this->validateCreate($graphs);
 
-		foreach ($graphs as $graph) {
-			$graph['graphid'] = $this->createReal($graph);
+		$this->createReal($graphs);
+		$this->inherit($graphs);
 
-			$this->inherit($graph);
-
-			$graphids[] = $graph['graphid'];
-		}
-
-		return ['graphids' => $graphids];
+		return ['graphids' => array_column($graphs, 'graphid')];
 	}
 
 	/**
-	 * Creates a new graph and returns it's ID.
+	 * Creates new graphs.
 	 *
-	 * @param $graph
-	 *
-	 * @return mixed
+	 * @param array $graphs
 	 */
-	protected function createReal($graph) {
-		$graphids = DB::insert('graphs', [$graph]);
-		$graphid = reset($graphids);
-		$sort_order = 0;
+	protected function createReal(array &$graphs) {
+		$graphids = DB::insert('graphs', $graphs);
+		$graph_items = [];
 
-		foreach ($graph['gitems'] as &$gitem) {
-			$gitem['graphid'] = $graphid;
+		// Collect graph_items to insert.
+		foreach ($graphs as $key => $graph) {
+			$sort_order = 0;
 
-			if (!array_key_exists('sortorder', $gitem)) {
-				$gitem['sortorder'] = $sort_order;
+			foreach ($graph['gitems'] as $graph_item) {
+				$graph_item['graphid'] = $graphids[$key];
+
+				if (!array_key_exists('sortorder', $graph_item)) {
+					$graph_item['sortorder'] = $sort_order;
+				}
+
+				$graph_items[] = $graph_item;
+
+				$sort_order++;
 			}
-
-			$sort_order++;
 		}
-		unset($gitem);
 
-		DB::insert('graphs_items', $graph['gitems']);
+		$graphs_itemsids = DB::insert('graphs_items', $graph_items);
 
-		return $graphid;
+		// Set id for graphs and graph items.
+		$i = 0;
+		foreach ($graphs as $key => &$graph) {
+			$graph['graphid'] = $graphids[$key];
+
+			foreach ($graph['gitems'] as &$graph_item) {
+				$graph_item['gitemid'] = $graphs_itemsids[$i++];
+			}
+			unset($graph_item);
+		}
+		unset($graph);
 	}
 
 	/**
-	 * Updates the graph if $graph differs from $dbGraph.
+	 * Updates the graphs.
 	 *
-	 * @param array $graph
-	 * @param array $dbGraph
+	 * @param array $graphs
 	 *
 	 * @return string
 	 */
-	protected function updateReal(array $graph, array $dbGraph) {
-		$dbGitems = zbx_toHash($dbGraph['gitems'], 'gitemid');
-		$dbGitemIds = zbx_toHash(zbx_objectValues($dbGitems, 'gitemid'));
+	protected function updateReal(array $graphs) {
+		$data = [];
+		foreach ($graphs as $graph) {
+			unset($graph['gitems']);
 
-		// update the graph if it's modified
-		if (DB::recordModified('graphs', $dbGraph, $graph)) {
-			DB::updateByPk($this->tableName(), $graph['graphid'], $graph);
+			$data[] = ['values' => $graph, 'where' => ['graphid' => $graph['graphid']]];
 		}
+		DB::update('graphs', $data);
 
-		// delete remaining items only if new items or items that require update are set
-		if ($graph['gitems']) {
-			$insertGitems = [];
-			$deleteGitemIds = $dbGitemIds;
+		$db_graph_items = API::GraphItem()->get([
+			'output' => ['gitemid', 'itemid', 'drawtype', 'sortorder', 'color', 'yaxisside', 'calc_fnc', 'type'],
+			'graphids' => array_column($graphs, 'graphid'),
+			'preservekeys' => true,
+			'nopermissions' => true
+		]);
+
+		$ins_graph_items = [];
+		$upd_graph_items = [];
+
+		foreach ($graphs as $graph) {
 			$sort_order = 0;
 
-			foreach ($graph['gitems'] as $gitem) {
-				// updating an existing item
-				if (!empty($gitem['gitemid']) && isset($dbGitemIds[$gitem['gitemid']])) {
-					if (DB::recordModified('graphs_items', $dbGitems[$gitem['gitemid']], $gitem)) {
-						DB::updateByPk('graphs_items', $gitem['gitemid'], $gitem);
+			foreach ($graph['gitems'] as $graph_item) {
+				// Update an existing item.
+				if (array_key_exists('gitemid', $graph_item)
+						&& array_key_exists($graph_item['gitemid'], $db_graph_items)) {
+					$db_graph_item = $db_graph_items[$graph_item['gitemid']];
+					$upd_graph_item = DB::getUpdatedValues('graphs_items', $graph_item, $db_graph_item);
+
+					if ($upd_graph_item) {
+						$upd_graph_items[] = [
+							'values' => $upd_graph_item,
+							'where' => ['gitemid' => $graph_item['gitemid']]
+						];
 					}
 
-					// remove this graph item from the collection so it won't get deleted
-					unset($deleteGitemIds[$gitem['gitemid']]);
+					unset($db_graph_items[$graph_item['gitemid']]);
 				}
-				// adding a new item
+				// Adding a new item.
 				else {
-					$gitem['graphid'] = $graph['graphid'];
+					$graph_item['graphid'] = $graph['graphid'];
 
-					if (!array_key_exists('sortorder', $gitem)) {
-						$gitem['sortorder'] = $sort_order;
+					if (!array_key_exists('sortorder', $graph_item)) {
+						$graph_item['sortorder'] = $sort_order;
 					}
 
+					$ins_graph_items[] = $graph_item;
+
 					$sort_order++;
-
-					$insertGitems[] = $gitem;
 				}
 			}
-
-			if ($deleteGitemIds) {
-				DB::delete('graphs_items', ['gitemid' => $deleteGitemIds]);
-			}
-
-			if ($insertGitems) {
-				DB::insert('graphs_items', $insertGitems);
-			}
 		}
 
-		return $graph['graphid'];
+		if ($ins_graph_items) {
+			DB::insert('graphs_items', $ins_graph_items);
+		}
+
+		if ($upd_graph_items) {
+			DB::update('graphs_items', $upd_graph_items);
+		}
+
+		if ($db_graph_items) {
+			DB::delete('graphs_items', ['gitemid' => array_keys($db_graph_items)]);
+		}
 	}
 
 	/**
@@ -850,4 +872,412 @@ abstract class CGraphGeneral extends CApiService {
 			}
 		}
 	}
+
+	/**
+	 * Returns visible host name. Can be used for error reporting.
+	 *
+	 * @static
+	 *
+	 * @param string|int $hostid
+	 *
+	 * @return string
+	 */
+	private static function getHostName($hostid): string {
+		return DB::select('hosts', [
+			'output' => ['name'],
+			'hostids' => $hostid
+		])[0]['name'];
+	}
+
+	/**
+	 * Adding graph items for selected graphs.
+	 *
+	 * @static
+	 *
+	 * @param array $graphs
+	 * @param bool  $with_hostid
+	 *
+	 * @return array
+	 */
+	private static function addGraphItems(array $graphs, bool $with_hostid = false): array {
+		$sql = $with_hostid
+			? 'SELECT gi.gitemid,gi.graphid,gi.itemid,i.hostid'.
+				' FROM graphs_items gi,items i'.
+				' WHERE gi.itemid=i.itemid'.
+					' AND '.dbConditionId('gi.graphid', array_keys($graphs)).
+				' ORDER BY gi.sortorder'
+			: 'SELECT gi.gitemid,gi.graphid,gi.itemid'.
+				' FROM graphs_items gi'.
+				' WHERE '.dbConditionId('gi.graphid', array_keys($graphs)).
+				' ORDER BY gi.sortorder';
+		$db_graph_items = DBselect($sql);
+
+		while ($db_graph_item = DBfetch($db_graph_items)) {
+			$graphid = $db_graph_item['graphid'];
+			unset($db_graph_item['graphid']);
+
+			$graphs[$graphid]['gitems'][] = $db_graph_item;
+		}
+
+		return $graphs;
+	}
+
+	/**
+	 * Updates the children of the graph on the given hosts and propagates the inheritance to the child hosts.
+	 *
+	 * @param array      $graphs   An array of graphs to inherit. Each graph must contain all graph properties including
+	 *                             "gitems" property.
+	 * @param array|null $hostids  An array of hosts to inherit to; if set to null, the graphs will be inherited to all
+	 *                             linked hosts or templates.
+	 * @throws APIException
+	 */
+	protected function inherit(array $graphs, array $hostids = null): void {
+		$graphs = array_column($graphs, null, 'graphid');
+
+		if ($hostids === null) {
+			/*
+			 * From the passed graphs we are able to inherit only those, which are template graphs and templates of
+			 * which are linked at least to one host. There we try to find the graphs which meet these conditions.
+			 */
+			$graphids = DBfetchColumn(DBselect(
+				'SELECT DISTINCT gi.graphid'.
+				' FROM graphs_items gi,items i,hosts h,hosts_templates ht,hosts h2'.
+				' WHERE gi.itemid=i.itemid'.
+					' AND i.hostid=h.hostid'.
+					' AND h.hostid=ht.templateid'.
+					' AND ht.hostid=h2.hostid'.
+					' AND '.dbConditionId('gi.graphid', array_keys($graphs)).
+					' AND h.status='.HOST_STATUS_TEMPLATE.
+					' AND '.dbConditionInt('h2.flags', [ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_CREATED])
+			), 'graphid');
+
+			// Based on the found graphs, we leave only graphs that is possible to inherit.
+			$graphs = array_intersect_key($graphs, array_flip($graphids));
+
+			if (!$graphs) {
+				return;
+			}
+		}
+
+		$same_name_graphs = [];
+		$itemids = [];
+
+		foreach ($graphs as $graphid => $graph) {
+			$same_name_graphs[$graph['name']][] = $graphid;
+
+			if ($graph['ymin_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE && $graph['ymin_itemid'] != 0) {
+				$itemids[$graph['ymin_itemid']] = true;
+			}
+
+			if ($graph['ymax_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE && $graph['ymax_itemid'] != 0) {
+				$itemids[$graph['ymax_itemid']] = true;
+			}
+
+			foreach ($graph['gitems'] as $gitem) {
+				$itemids[$gitem['itemid']] = true;
+			}
+		}
+
+		$itemids = array_keys($itemids);
+
+		$graph_templateids = [];
+		$templateids = [];
+
+		$db_graph_templates = DBselect(
+			'SELECT DISTINCT gi.graphid,i.hostid'.
+			' FROM graphs_items gi,items i'.
+			' WHERE gi.itemid=i.itemid'.
+				' AND '.dbConditionId('gi.graphid', array_keys($graphs))
+		);
+
+		while ($db_graph_template = DBfetch($db_graph_templates)) {
+			$graph_templateids[$db_graph_template['graphid']] = $db_graph_template['hostid'];
+			$templateids[$db_graph_template['hostid']] = true;
+		}
+
+		$templateids_hosts = [];
+
+		$sql = 'SELECT ht.templateid,ht.hostid'.
+			' FROM hosts_templates ht,hosts h2'.
+			' WHERE ht.hostid=h2.hostid'.
+				' AND '.dbConditionId('ht.templateid', array_keys($templateids)).
+				' AND '.dbConditionInt('h2.flags', [ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_CREATED]);
+		if ($hostids !== null) {
+			$sql .= ' AND '.dbConditionId('ht.hostid', $hostids);
+		}
+		$db_host_templates = DBselect($sql);
+
+		while ($db_host_template = DBfetch($db_host_templates)) {
+			$templateids_hosts[$db_host_template['templateid']][$db_host_template['hostid']] = true;
+		}
+
+		foreach ($same_name_graphs as $name => $graphids) {
+			if (count($graphids) > 1) {
+				$_templateids = [];
+
+				foreach ($graphids as $graphid) {
+					$_templateids[] = $graph_templateids[$graphid];
+				}
+
+				$_templateids_count = count($_templateids);
+
+				for ($i = 0; $i < $_templateids_count - 1; $i++) {
+					for ($j = $i + 1; $j < $_templateids_count; $j++) {
+						$same_hosts = array_intersect_key($templateids_hosts[$_templateids[$i]],
+							$templateids_hosts[$_templateids[$j]]
+						);
+
+						if ($same_hosts) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Graph "%1$s" already exists on "%2$s".', $name, self::getHostName(key($same_hosts)))
+							);
+						}
+					}
+				}
+			}
+		}
+
+		/*
+		 * In case when all equivalent items to graphs templates items exists on all hosts, to which they are linked,
+		 * there will be collected relations between template items and these equivalents on hosts.
+		 */
+		$item_links = [];
+
+		$sql = 'SELECT src.itemid AS src_itemid,dest.itemid AS dest_itemid,dest.hostid AS dest_hostid'.
+			' FROM items src,items dest'.
+			' WHERE src.itemid=dest.templateid'.
+				' AND '.dbConditionId('src.itemid', $itemids);
+		if ($hostids !== null) {
+			$sql .= ' AND '.dbConditionId('dest.hostid', $hostids);
+		}
+		$db_items = DBselect($sql);
+
+		while ($db_item = DBfetch($db_items)) {
+			$item_links[$db_item['src_itemid']][$db_item['dest_hostid']] = $db_item['dest_itemid'];
+		}
+
+		$chd_graphs_tpl = [];
+		$chd_graphs_name = [];
+
+		// Preparing list of child graphs by graph templateid.
+		$sql = 'SELECT DISTINCT g.graphid,g.name,g.templateid,i.hostid'.
+			' FROM graphs g,graphs_items gi,items i'.
+			' WHERE g.graphid=gi.graphid'.
+				' AND gi.itemid=i.itemid'.
+				' AND '.dbConditionId('g.templateid', array_keys($graphs));
+		if ($hostids !== null) {
+			$sql .= ' AND '.dbConditionId('i.hostid', $hostids);
+		}
+		$chd_graphs = DBfetchArrayAssoc(DBselect($sql), 'graphid');
+
+		if ($chd_graphs) {
+			$chd_graphs = self::addGraphItems($chd_graphs);
+
+			foreach ($chd_graphs as $chd_graph) {
+				$chd_graphs_tpl[$chd_graph['hostid']][$chd_graph['templateid']] = array_intersect_key($chd_graph,
+					array_flip(['graphid', 'name', 'gitems'])
+				);
+			}
+		}
+
+		$hostids_by_name = [];
+
+		// Preparing list of child graphs by graph name.
+		foreach ($graph_templateids as $graphid => $templateid) {
+			foreach (array_keys($templateids_hosts[$templateid]) as $hostid) {
+				if (!array_key_exists($hostid, $chd_graphs_tpl)
+						|| !array_key_exists($graphid, $chd_graphs_tpl[$hostid])) {
+					$hostids_by_name[$graphs[$graphid]['name']][] = $hostid;
+				}
+			}
+		}
+
+		$chd_graphs = [];
+
+		foreach ($hostids_by_name as $name => $_hostids) {
+			$flags = $this instanceof CGraph
+					? [ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_CREATED]
+					: [ZBX_FLAG_DISCOVERY_PROTOTYPE];
+			$sql = 'SELECT g.graphid,g.name,g.templateid,g.flags'.
+				' FROM graphs g'.
+				' WHERE '.dbConditionString('g.name', [$name]).
+					' AND '.dbConditionInt('g.flags', $flags).
+					' AND EXISTS ('.
+						'SELECT NULL'.
+						' FROM graphs_items gi,items i'.
+						' WHERE g.graphid=gi.graphid'.
+							' AND gi.itemid=i.itemid'.
+							' AND '.dbConditionId('i.hostid', $_hostids).
+					')';
+			$chd_graphs += DBfetchArrayAssoc(DBselect($sql), 'graphid');
+		}
+
+		if ($chd_graphs) {
+			$chd_graphs = self::addGraphItems($chd_graphs, true);
+
+			foreach ($chd_graphs as $chd_graph) {
+				$hostid = $chd_graph['gitems'][0]['hostid'];
+
+				if ($chd_graph['templateid'] != 0) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _s(
+						'Graph "%1$s" already exists on "%2$s" (inherited from another template).', $chd_graph['name'],
+						self::getHostName($hostid)
+					));
+				}
+				elseif ($this instanceof CGraph && $chd_graph['flags'] & ZBX_FLAG_DISCOVERY_CREATED) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _s(
+						'Graph "%1$s" already exists on "%2$s" as a graph created from graph prototype.',
+						$chd_graph['name'], self::getHostName($hostid)
+					));
+				}
+
+				$chd_graphs_name[$hostid][$chd_graph['name']] = array_intersect_key($chd_graph,
+					array_flip(['graphid', 'name', 'gitems'])
+				);
+			}
+		}
+
+		$ins_graphs = [];
+		$upd_graphs = [];
+		$upd_hostids_by_name = [];
+
+		foreach ($graphs as $graphid => $graph) {
+			$templateid = $graph_templateids[$graphid];
+
+			foreach (array_keys($templateids_hosts[$templateid]) as $hostid) {
+				$chd_graph = null;
+
+				if (array_key_exists($hostid, $chd_graphs_tpl)
+						&& array_key_exists($graphid, $chd_graphs_tpl[$hostid])) {
+					$chd_graph = $chd_graphs_tpl[$hostid][$graphid];
+
+					/*
+					 * If template graph name was changed, we collect all that names to check whether graphs with the
+					 * same name already exists on child hosts/templates.
+					 */
+					if ($graph['name'] !== $chd_graph['name']) {
+						$upd_hostids_by_name[$graph['name']][] = $hostid;
+					}
+
+					$_graph = ['graphid' => $chd_graph['graphid'], 'templateid' => $graphid] + $graph;
+				}
+				elseif (array_key_exists($hostid, $chd_graphs_name)
+						&& array_key_exists($graph['name'], $chd_graphs_name[$hostid])) {
+					$chd_graph = $chd_graphs_name[$hostid][$graph['name']];
+					$chd_graph_itemids = array_column($chd_graph['gitems'], 'itemid');
+
+					if (count($graph['gitems']) !== count($chd_graph['gitems'])) {
+						self::exception(ZBX_API_ERROR_PARAMETERS, _s(
+							'Graph "%1$s" already exists on "%2$s" (items are not identical).', $graph['name'],
+							self::getHostName($hostid)
+						));
+					}
+
+					foreach ($graph['gitems'] as $gitem) {
+						$index = array_search($item_links[$gitem['itemid']][$hostid], $chd_graph_itemids);
+
+						if ($index === false) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _s(
+								'Graph "%1$s" already exists on "%2$s" (items are not identical).', $graph['name'],
+								self::getHostName($hostid)
+							));
+						}
+
+						unset($chd_graph_itemids[$index]);
+					}
+
+					$_graph = ['graphid' => $chd_graph['graphid'], 'templateid' => $graphid] + $graph;
+				}
+				else {
+					$_graph = ['templateid' => $graphid] + array_diff_key($graph, ['graphid' => true]);
+				}
+
+				$_graph['uuid'] = '';
+
+				if ($_graph['ymin_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE && $_graph['ymin_itemid'] != 0) {
+					$_graph['ymin_itemid'] = $item_links[$_graph['ymin_itemid']][$hostid];
+				}
+
+				if ($_graph['ymax_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE && $_graph['ymax_itemid'] != 0) {
+					$_graph['ymax_itemid'] = $item_links[$_graph['ymax_itemid']][$hostid];
+				}
+
+				CArrayHelper::sort($_graph['gitems'], ['sortorder']);
+
+				foreach ($_graph['gitems'] as &$gitem) {
+					$gitem['itemid'] = $item_links[$gitem['itemid']][$hostid];
+
+					if ($chd_graph !== null && $chd_graph['gitems']) {
+						$gitem['gitemid'] = array_shift($chd_graph['gitems'])['gitemid'];
+					}
+				}
+				unset($gitem);
+
+				if ($chd_graph !== null) {
+					$upd_graphs[] = $_graph;
+				}
+				else {
+					$ins_graphs[] = $_graph;
+				}
+			}
+		}
+
+		// Check if graph with a new name already exists on the child host.
+		if ($upd_hostids_by_name) {
+			$sql_where = [];
+			foreach ($upd_hostids_by_name as $name => $_hostids) {
+				$sql_where[] = dbConditionInt('i.hostid', $_hostids).' AND g.name='.zbx_dbstr($name);
+			}
+
+			$sql = 'SELECT i.hostid,g.name'.
+				' FROM graphs g,graphs_items gi,items i'.
+				' WHERE g.graphid=gi.graphid'.
+					' AND gi.itemid=i.itemid'.
+					' AND (('.implode(') OR (', $sql_where).'))';
+			$db_graphs = DBselect($sql, 1);
+
+			if ($db_graph = DBfetch($db_graphs)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" already exists on "%2$s".',
+					$db_graph['name'], self::getHostName($db_graph['hostid'])
+				));
+			}
+		}
+
+		if ($ins_graphs) {
+			$this->createReal($ins_graphs);
+		}
+
+		if ($upd_graphs) {
+			$this->updateReal($upd_graphs);
+		}
+
+		$this->inherit(array_merge($ins_graphs + $upd_graphs));
+	}
+
+	/**
+	 * Inherit template graphs from template to host.
+	 *
+	 * @param array $data
+	 */
+	public function syncTemplates(array $data): void {
+		$output = ['graphid', 'name', 'width', 'height', 'yaxismin', 'yaxismax', 'templateid', 'show_work_period',
+			'show_triggers', 'graphtype', 'show_legend', 'show_3d', 'percent_left', 'percent_right', 'ymin_type',
+			'ymax_type', 'ymin_itemid', 'ymax_itemid'
+		];
+
+		if ($this instanceof CGraphPrototype) {
+			$output[] = 'discover';
+		}
+
+		$graphs = $this->get([
+			'output' => $output,
+			'selectGraphItems' => ['itemid', 'drawtype', 'sortorder', 'color', 'yaxisside', 'calc_fnc', 'type'],
+			'hostids' => $data['templateids'],
+			'preservekeys' => true
+		]);
+
+		if ($graphs) {
+			$this->inherit($graphs, $data['hostids']);
+		}
+	}
 }
