@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,6 +28,13 @@
  */
 class CIconMap extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_SUPER_ADMIN]
+	];
+
 	protected $tableName = 'icon_map';
 	protected $tableAlias = 'im';
 	protected $sortColumns = ['iconmapid', 'name'];
@@ -114,7 +121,7 @@ class CIconMap extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$dbRes = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$dbRes = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($iconMap = DBfetch($dbRes)) {
 			if ($options['countOutput']) {
 				$result = $iconMap['rowscount'];
@@ -183,7 +190,7 @@ class CIconMap extends CApiService {
 			'default_iconid' =>	['type' => API_ID, 'flags' => API_REQUIRED],
 			'mappings' =>		['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_REQUIRED, 'uniq' => [['inventory_link', 'expression']], 'fields' => [
 				'iconid' =>			['type' => API_ID, 'flags' => API_REQUIRED],
-				'expression' =>		['type' => API_REGEX, 'flags' => API_NOT_EMPTY | API_REQUIRED, 'length' => DB::getFieldLength('icon_mapping', 'expression')],
+				'expression' =>		['type' => API_REGEX, 'flags' => API_NOT_EMPTY | API_REQUIRED | API_ALLOW_GLOBAL_REGEX, 'length' => DB::getFieldLength('icon_mapping', 'expression')],
 				'inventory_link' =>	['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => '1:70'],
 				'sortorder' =>		['type' => API_INT32, 'flags' => API_DEPRECATED]
 			]]
@@ -255,7 +262,7 @@ class CIconMap extends CApiService {
 			'default_iconid' =>	['type' => API_ID],
 			'mappings' =>		['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY, 'uniq' => [['inventory_link', 'expression']], 'fields' => [
 				'iconid' =>			['type' => API_ID, 'flags' => API_REQUIRED],
-				'expression' =>		['type' => API_REGEX, 'flags' => API_NOT_EMPTY | API_REQUIRED, 'length' => DB::getFieldLength('icon_mapping', 'expression')],
+				'expression' =>		['type' => API_REGEX, 'flags' => API_NOT_EMPTY | API_REQUIRED | API_ALLOW_GLOBAL_REGEX, 'length' => DB::getFieldLength('icon_mapping', 'expression')],
 				'inventory_link' =>	['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => '1:70'],
 				'sortorder' =>		['type' => API_INT32, 'flags' => API_DEPRECATED]
 			]]
@@ -308,7 +315,7 @@ class CIconMap extends CApiService {
 		]);
 
 		if ($db_iconmaps) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Icon map "%s" already exists.', $db_iconmaps[0]['name']));
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Icon map "%1$s" already exists.', $db_iconmaps[0]['name']));
 		}
 	}
 
@@ -325,8 +332,12 @@ class CIconMap extends CApiService {
 		foreach ($iconmaps as $iconmap) {
 			if (array_key_exists('mappings', $iconmap)) {
 				foreach ($iconmap['mappings'] as $mapping) {
-					if ($mapping['expression'][0] == '@') {
-						$names[substr($mapping['expression'], 1)] = true;
+					if ($mapping['expression'][0] === '@') {
+						/*
+						 * substr() performs differently on PHP < 7. Prior to PHP 7 version, FALSE was returned if
+						 * string is equal to start characters long. As of PHP 7, an empty string will be returned.
+						 */
+						$names[isset($mapping['expression'][1]) ? substr($mapping['expression'], 1) : ''] = true;
 					}
 				}
 			}
