@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ class DB {
 	const FIELD_TYPE_UINT = 'uint';
 	const FIELD_TYPE_BLOB = 'blob';
 	const FIELD_TYPE_TEXT = 'text';
+	const FIELD_TYPE_NCLOB = 'nclob';
 
 	private static $schema = null;
 
@@ -65,9 +66,6 @@ class DB {
 				case ZBX_DB_ORACLE:
 					self::$dbBackend = new OracleDbBackend();
 					break;
-				case ZBX_DB_DB2:
-					self::$dbBackend = new Db2DbBackend();
-					break;
 			}
 		}
 
@@ -269,31 +267,16 @@ class DB {
 		$schema = self::getSchema($table_name);
 
 		if ($schema['fields'][$field_name]['type'] == self::FIELD_TYPE_TEXT) {
-			return ($DB['TYPE'] == ZBX_DB_DB2 || $DB['TYPE'] == ZBX_DB_ORACLE) ? 2048 : 65535;
+			return ($DB['TYPE'] == ZBX_DB_ORACLE) ? 2048 : 65535;
+		}
+
+		if ($schema['fields'][$field_name]['type'] == self::FIELD_TYPE_NCLOB) {
+			return 65535;
 		}
 
 		return $schema['fields'][$field_name]['length'];
 	}
 
-	private static function addMissingFields($tableSchema, $values) {
-		global $DB;
-
-		if ($DB['TYPE'] == ZBX_DB_MYSQL) {
-			foreach ($tableSchema['fields'] as $name => $field) {
-				if ($field['type'] == self::FIELD_TYPE_TEXT && !$field['null']) {
-					foreach ($values as &$value) {
-						if (!isset($value[$name])) {
-							$value[$name] = '';
-						}
-					}
-					unset($value);
-				}
-			}
-		}
-
-		return $values;
-	}
-
 	public static function getDefaults($table) {
 		$table = self::getSchema($table);
 
@@ -321,9 +304,55 @@ class DB {
 		return isset($field['default']) ? $field['default'] : null;
 	}
 
-	public static function checkValueTypes($table, &$values) {
+	/**
+	 * Get the updated values of a record by correctly comparing the new and old ones, taking field types into account.
+	 *
+	 * @param string $table_name
+	 * @param array  $new_values
+	 * @param array  $old_values
+	 *
+	 * @return array
+	 */
+	public static function getUpdatedValues(string $table_name, array $new_values, array $old_values): array {
+		$updated_values = [];
+
+		// Discard field names not existing in the target table.
+		$fields = array_intersect_key(DB::getSchema($table_name)['fields'], $new_values);
+
+		foreach ($fields as $name => $spec) {
+			if (!array_key_exists($name, $old_values)) {
+				$updated_values[$name] = $new_values[$name];
+				continue;
+			}
+
+			switch ($spec['type']) {
+				case DB::FIELD_TYPE_ID:
+					if (bccomp($new_values[$name], $old_values[$name]) != 0) {
+						$updated_values[$name] = $new_values[$name];
+					}
+					break;
+
+				case DB::FIELD_TYPE_INT:
+				case DB::FIELD_TYPE_UINT:
+				case DB::FIELD_TYPE_FLOAT:
+					if ($new_values[$name] != $old_values[$name]) {
+						$updated_values[$name] = $new_values[$name];
+					}
+					break;
+
+				default:
+					if ($new_values[$name] !== $old_values[$name]) {
+						$updated_values[$name] = $new_values[$name];
+					}
+					break;
+			}
+		}
+
+		return $updated_values;
+	}
+
+	private static function checkValueTypes($tableSchema, &$values) {
 		global $DB;
-		$tableSchema = self::getSchema($table);
 
 		foreach ($values as $field => $value) {
 			if (!isset($tableSchema['fields'][$field])) {
@@ -381,21 +410,51 @@ class DB {
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
 					case self::FIELD_TYPE_TEXT:
-						if ($DB['TYPE'] == ZBX_DB_DB2 || $DB['TYPE'] == ZBX_DB_ORACLE) {
+						if ($DB['TYPE'] == ZBX_DB_ORACLE) {
 							$length = mb_strlen($values[$field]);
 
 							if ($length > 2048) {
-								self::exception(self::SCHEMA_ERROR, _s('Value "%1$s" is too long for field "%2$s" - %3$d characters. Allowed length is 2048 characters.',
-									$values[$field], $field, $length));
+								self::exception(self::SCHEMA_ERROR, _s('Value "%1$s" is too long for field "%2$s" - %3$d characters. Allowed length is %4$d characters.',
+									$values[$field], $field, $length, 2048));
 							}
 						}
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
+					case self::FIELD_TYPE_NCLOB:
+						// Using strlen because 4000 bytes is largest possible string literal in oracle query.
+						if ($DB['TYPE'] == ZBX_DB_ORACLE && strlen($values[$field]) > ORACLE_MAX_STRING_SIZE) {
+							$chunks = zbx_dbstr(self::chunkMultibyteStr($values[$field], ORACLE_MAX_STRING_SIZE));
+							$values[$field] = 'TO_NCLOB('.implode(') || TO_NCLOB(', $chunks).')';
+						}
+						else {
+							$values[$field] = zbx_dbstr($values[$field]);
+						}
+						break;
 				}
 			}
 		}
 	}
 
+	/**
+	 * @param string $str
+	 * @param int $chunk_size
+	 *
+	 * @return array
+	 */
+	public static function chunkMultibyteStr(string $str, int $chunk_size): array {
+		$chunks = [];
+		$offset = 0;
+		$size = strlen($str);
+
+		while ($offset < $size) {
+			$chunk = mb_strcut($str, $offset, $chunk_size);
+			$chunks[] = $chunk;
+			$offset = strlen($chunk) + $offset;
+		}
+
+		return $chunks;
+	}
+
 	/**
 	 * Returns the records that match the given criteria.
 	 *
@@ -438,38 +497,48 @@ class DB {
 	 * @return array    an array of ids with the keys preserved
 	 */
 	public static function insert($table, $values, $getids = true) {
-		if (empty($values)) {
-			return true;
+		$table_schema = self::getSchema($table);
+		$fields = array_reduce($values, 'array_merge', []);
+		$fields = array_intersect_key($fields, $table_schema['fields']);
+
+		foreach ($fields as $field => &$value) {
+			$value = array_key_exists('default', $table_schema['fields'][$field])
+				? $table_schema['fields'][$field]['default']
+				: null;
 		}
+		unset($value);
 
-		$resultIds = [];
-
-		if ($getids) {
-			$id = self::reserveIds($table, count($values));
+		foreach ($values as $key => &$row) {
+			$row = array_merge($fields, $row);
 		}
+		unset($row);
 
-		$tableSchema = self::getSchema($table);
+		return self::insertBatch($table, $values, $getids);
+	}
 
-		$values = self::addMissingFields($tableSchema, $values);
+	/**
+	 * Returns the list of mandatory fields with default values for INSERT statements.
+	 *
+	 * @static
+	 *
+	 * @param array $table_schema
+	 *
+	 * @return array
+	 */
+	private static function getMandatoryFields(array $table_schema): array {
+		global $DB;
 
-		foreach ($values as $key => $row) {
-			if ($getids) {
-				$resultIds[$key] = $id;
-				$row[$tableSchema['key']] = $id;
-				$id = bcadd($id, 1, 0);
-			}
+		$mandatory_fields = [];
 
-			self::checkValueTypes($table, $row);
-
-			$sql = 'INSERT INTO '.$table.' ('.implode(',', array_keys($row)).')'.
-					' VALUES ('.implode(',', array_values($row)).')';
-
-			if (!DBexecute($sql)) {
-				self::exception(self::DBEXECUTE_ERROR, _s('SQL statement execution has failed "%1$s".', $sql));
+		if ($DB['TYPE'] == ZBX_DB_MYSQL) {
+			foreach ($table_schema['fields'] as $name => $field) {
+				if ($field['type'] == self::FIELD_TYPE_TEXT || $field['type'] == self::FIELD_TYPE_NCLOB) {
+					$mandatory_fields += [$name => $field['default']];
+				}
 			}
 		}
 
-		return $resultIds;
+		return $mandatory_fields;
 	}
 
 	/**
@@ -488,28 +557,28 @@ class DB {
 
 		$resultIds = [];
 
-		$tableSchema = self::getSchema($table);
-		$values = self::addMissingFields($tableSchema, $values);
+		$table_schema = self::getSchema($table);
 
 		if ($getids) {
 			$id = self::reserveIds($table, count($values));
 		}
 
-		$newValues = [];
-		foreach ($values as $key => $row) {
+		$mandatory_fields = self::getMandatoryFields($table_schema);
+
+		foreach ($values as $key => &$row) {
 			if ($getids) {
 				$resultIds[$key] = $id;
-				$row[$tableSchema['key']] = $id;
-				$values[$key][$tableSchema['key']] = $id;
+				$row[$table_schema['key']] = $id;
 				$id = bcadd($id, 1, 0);
 			}
-			self::checkValueTypes($table, $row);
-			$newValues[] = $row;
+
+			$row += $mandatory_fields;
+
+			self::checkValueTypes($table_schema, $row);
 		}
+		unset($row);
 
-		$fields = array_keys(reset($newValues));
-
-		$sql = self::getDbBackend()->createInsertQuery($table, $fields, $newValues);
+		$sql = self::getDbBackend()->createInsertQuery($table, array_keys(reset($values)), $values);
 
 		if (!DBexecute($sql)) {
 			self::exception(self::DBEXECUTE_ERROR, _s('SQL statement execution has failed "%1$s".', $sql));
@@ -538,7 +607,7 @@ class DB {
 		$data = zbx_toArray($data);
 		foreach ($data as $row) {
 			// check
-			self::checkValueTypes($table, $row['values']);
+			self::checkValueTypes($tableSchema, $row['values']);
 			if (empty($row['values'])) {
 				self::exception(self::DBEXECUTE_ERROR, _s('Cannot perform update statement on table "%1$s" without values.', $table));
 			}
@@ -813,11 +882,11 @@ class DB {
 	 * Delete data from DB.
 	 *
 	 * Example:
-	 * DB::delete('applications', array('applicationid'=>array(1, 8, 6)));
-	 * DELETE FROM applications WHERE applicationid IN (1, 8, 6)
+	 * DB::delete('items', ['itemid' => [1, 8, 6]]);
+	 * DELETE FROM items WHERE itemid IN (1, 8, 6)
 	 *
-	 * DB::delete('applications', array('applicationid'=>array(1), 'templateid'=array(10)));
-	 * DELETE FROM applications WHERE applicationid IN (1) AND templateid IN (10)
+	 * DB::delete('items', ['itemid' => [1], 'templateid' => [10]]);
+	 * DELETE FROM items WHERE itemid IN (1) AND templateid IN (10)
 	 *
 	 * @param string $table
 	 * @param array  $wheres pair of fieldname => fieldvalues
@@ -862,6 +931,9 @@ class DB {
 			'output' => [],
 			'countOutput' => false,
 			'filter' => [],
+			'search' => [],
+			'startSearch' => false,
+			'searchByAny' => false,
 			'sortfield' => [],
 			'sortorder' => [],
 			'limit' => null,
@@ -962,6 +1034,9 @@ class DB {
 		// add filter options
 		$sql_parts = self::applyQueryFilterOptions($table_name, $options, $table_alias, $sql_parts);
 
+		// add search options
+		$sql_parts = self::applyQuerySearchOptions($table_name, $options, $table_alias, $sql_parts);
+
 		// add sort options
 		$sql_parts = self::applyQuerySortOptions($table_name, $options, $table_alias, $sql_parts);
 
@@ -1031,12 +1106,12 @@ class DB {
 
 			switch ($field_schema['type']) {
 				case self::FIELD_TYPE_ID:
-					$sql_parts['where'][] = dbConditionId($field_name, $options[$pk_option], false, true, false);
+					$sql_parts['where'][] = dbConditionId($field_name, $options[$pk_option]);
 					break;
 
 				case self::FIELD_TYPE_INT:
 				case self::FIELD_TYPE_UINT:
-					$sql_parts['where'][] = dbConditionInt($field_name, $options[$pk_option], false, true, false);
+					$sql_parts['where'][] = dbConditionInt($field_name, $options[$pk_option]);
 					break;
 
 				default:
@@ -1052,6 +1127,79 @@ class DB {
 		return $sql_parts;
 	}
 
+	/**
+	 * Modifies the SQL parts to implement all of the search related options.
+	 *
+	 * @param string $table_name
+	 * @param array  $options
+	 * @param array  $options['search']
+	 * @param bool   $options['startSearch']
+	 * @param bool   $options['searchByAny']
+	 * @param string $table_alias
+	 * @param array  $sql_parts
+	 *
+	 * @return array
+	 */
+	private static function applyQuerySearchOptions($table_name, array $options, $table_alias = null,
+			array $sql_parts) {
+		global $DB;
+
+		$table_schema = DB::getSchema($table_name);
+		$unsupported_types = [self::FIELD_TYPE_INT, self::FIELD_TYPE_ID, self::FIELD_TYPE_FLOAT, self::FIELD_TYPE_UINT,
+			self::FIELD_TYPE_BLOB
+		];
+
+		$start = $options['startSearch'] ? '' : '%';
+		$glue = $options['searchByAny'] ? ' OR ' : ' AND ';
+
+		$search = [];
+
+		foreach ($options['search'] as $field_name => $patterns) {
+			if (!array_key_exists($field_name, $table_schema['fields'])) {
+				self::exception(self::SCHEMA_ERROR,
+					vsprintf('%s: field "%s.%s" does not exist.', [__FUNCTION__, $table_name, $field_name])
+				);
+			}
+
+			$field_schema = $table_schema['fields'][$field_name];
+
+			if (in_array($field_schema['type'], $unsupported_types)) {
+				self::exception(self::SCHEMA_ERROR,
+					vsprintf('%s: field "%s.%s" has an unsupported type.', [__FUNCTION__, $table_name, $field_name])
+				);
+			}
+
+			if ($patterns === null) {
+				continue;
+			}
+
+			foreach ((array) $patterns as $pattern) {
+				// escaping parameter that is about to be used in LIKE statement
+				$pattern = mb_strtoupper(strtr($pattern, ['!' => '!!', '%' => '!%', '_' => '!_']));
+				$pattern = $start.$pattern.'%';
+
+				if ($DB['TYPE'] == ZBX_DB_ORACLE && $field_schema['type'] === DB::FIELD_TYPE_NCLOB
+						&& strlen($pattern) > ORACLE_MAX_STRING_SIZE) {
+					$chunks = zbx_dbstr(DB::chunkMultibyteStr($pattern, ORACLE_MAX_STRING_SIZE));
+					$pattern = 'TO_NCLOB('.implode(') || TO_NCLOB(', $chunks).')';
+				}
+				else {
+					$pattern = zbx_dbstr($pattern);
+				}
+
+				$search[] = 'UPPER('.self::fieldId($field_name, $table_alias).') LIKE '.$pattern." ESCAPE '!'";
+			}
+		}
+
+		if ($search) {
+			$sql_parts['where'][] = ($options['searchByAny'] && count($search) > 1)
+				? '('.implode($glue, $search).')'
+				: implode($glue, $search);
+		}
+
+		return $sql_parts;
+	}
+
 	/**
 	 * Apply filter conditions to sql built query.
 	 *
@@ -1075,7 +1223,7 @@ class DB {
 
 			$field_schema = $table_schema['fields'][$field_name];
 
-			if ($field_schema['type'] == self::FIELD_TYPE_TEXT) {
+			if ($field_schema['type'] == self::FIELD_TYPE_TEXT || $field_schema['type'] == self::FIELD_TYPE_NCLOB) {
 				self::exception(self::SCHEMA_ERROR,
 					vsprintf('%s: field "%s.%s" has an unsupported type.', [__FUNCTION__, $table_name, $field_name])
 				);
@@ -1091,12 +1239,12 @@ class DB {
 
 			switch ($field_schema['type']) {
 				case self::FIELD_TYPE_ID:
-					$filter[] = dbConditionId(self::fieldId($field_name, $table_alias), $value, false, true, false);
+					$filter[] = dbConditionId(self::fieldId($field_name, $table_alias), $value);
 					break;
 
 				case self::FIELD_TYPE_INT:
 				case self::FIELD_TYPE_UINT:
-					$filter[] = dbConditionInt(self::fieldId($field_name, $table_alias), $value, false, true, false);
+					$filter[] = dbConditionInt(self::fieldId($field_name, $table_alias), $value);
 					break;
 
 				default:
