@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,27 +22,14 @@
 /**
  * Base XML validation class.
  */
-class CXmlValidator {
+class CXmlValidator extends CXmlValidatorGeneral {
 
-	/**
-	 * Accepted versions.
-	 *
-	 * @var CXmlValidator[]
-	 */
-	protected $versionValidators = [];
+	private $factory;
 
-	public function __construct() {
-		$this->versionValidators = [
-			'1.0' => 'C10XmlValidator',
-			'2.0' => 'C20XmlValidator',
-			'3.0' => 'C30XmlValidator',
-			'3.2' => 'C32XmlValidator',
-			'3.4' => 'C34XmlValidator',
-			'4.0' => 'C40XmlValidator',
-			'4.2' => 'C42XmlValidator',
-			'4.4' => 'C44XmlValidator',
-			'5.0' => 'C50XmlValidator'
-		];
+	public function __construct(CRegistryFactory $factory, string $format) {
+		parent::__construct($format);
+
+		$this->factory = $factory;
 	}
 
 	/**
@@ -56,23 +43,35 @@ class CXmlValidator {
 	 * @return array  Validator does some manipulations for the incoming data. For example, converts empty tags to an
 	 *                array, if desired. Converted array is returned.
 	 */
-	public function validate(array $data, $format) {
+	public function validate(array $data, string $path): array {
 		$rules = ['type' => XML_ARRAY, 'rules' => [
 			'zabbix_export' => ['type' => XML_ARRAY | XML_REQUIRED, 'check_unexpected' => false, 'rules' => [
 				'version' => ['type' => XML_STRING | XML_REQUIRED]
 			]]
 		]];
 
-		$data = (new CXmlValidatorGeneral($rules, $format))->validate($data, '/');
+		$strict = $this->getStrict();
+		$is_preview = $this->isPreview();
+
+		$data = $this
+			->setStrict(true)
+			->doValidate($rules, $data, $path);
+
 		$version = $data['zabbix_export']['version'];
 
-		if (!array_key_exists($version, $this->versionValidators)) {
+		if (!$this->factory->hasObject($version)) {
 			throw new Exception(
 				_s('Invalid tag "%1$s": %2$s.', '/zabbix_export/version', _('unsupported version number'))
 			);
 		}
 
-		$data['zabbix_export'] = (new $this->versionValidators[$version]($format))
+		if (in_array($version, ['1.0', '2.0', '3.0', '3.2', '3.4', '4.0', '4.2', '4.4', '5.0', '5.2'])) {
+			unset($data['zabbix_export']['screens']);
+		}
+
+		$data['zabbix_export'] = $this->factory->getObject($version)
+			->setStrict($strict)
+			->setPreview($is_preview)
 			->validate($data['zabbix_export'], '/zabbix_export');
 
 		return $data;
