@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 
 
 /**
- * Setup wizzard form.
+ * Setup wizard form.
  */
 class CSetupWizard extends CForm {
 
@@ -181,7 +181,10 @@ class CSetupWizard extends CForm {
 		$setup_title = (new CDiv([new CSpan(_('Welcome to')), 'Zabbix '.$version[0]]))->addClass(ZBX_STYLE_SETUP_TITLE);
 
 		$default_lang = $this->getConfig('default_lang');
-		$lang_combobox = (new CComboBox('default_lang', $default_lang, 'submit();'))
+		$lang_select = (new CSelect('default_lang'))
+			->setId('default-lang')
+			->setValue($default_lang)
+			->setFocusableElementId('label-default-lang')
 			->setAttribute('autofocus', 'autofocus');
 
 		$all_locales_available = 1;
@@ -199,7 +202,7 @@ class CSetupWizard extends CForm {
 					|| setlocale(LC_MONETARY, zbx_locale_variants($localeid))
 			);
 
-			$lang_combobox->addItem($localeid, $locale['name'], null, $locale_available);
+			$lang_select->addOption((new CSelectOption($localeid, $locale['name']))->setDisabled(!$locale_available));
 
 			$all_locales_available &= (int) $locale_available;
 		}
@@ -210,18 +213,20 @@ class CSetupWizard extends CForm {
 		$language_error = '';
 		if (!function_exists('bindtextdomain')) {
 			$language_error = 'Translations are unavailable because the PHP gettext module is missing.';
-			$lang_combobox->setEnabled(false);
+			$lang_select->setReadonly();
 		}
 		elseif ($all_locales_available == 0) {
 			$language_error = _('You are not able to choose some of the languages, because locales for them are not installed on the web server.');
 		}
 
+		$language_error = ($language_error !== '')
+			? (makeErrorIcon($language_error))->addStyle('margin-left: 5px;')
+			: null;
+
 		$language_select = (new CFormList())
-			->addRow(_('Default language'),
-				($language_error !== '')
-					? [$lang_combobox, (makeErrorIcon($language_error))->addStyle('margin-left: 5px;')]
-					: $lang_combobox
-			);
+			->addRow(new CLabel(_('Default language'), $lang_select->getFocusableElementId()), [
+				$lang_select, $language_error
+			]);
 
 		return [(new CDiv([$setup_title, $language_select]))->addClass(ZBX_STYLE_SETUP_RIGHT_BODY)];
 	}
@@ -264,7 +269,7 @@ class CSetupWizard extends CForm {
 		}
 
 		if ($finalResult == CFrontendSetup::CHECK_FATAL) {
-			$message_box = makeMessageBox(false, $messages, null, false, true);
+			$message_box = makeMessageBox(ZBX_STYLE_MSG_BAD, $messages, null, false, true);
 		}
 		else {
 			$message_box = null;
@@ -286,8 +291,12 @@ class CSetupWizard extends CForm {
 				(new CVar('verify_host', 0))->removeId()
 			]);
 
-		$table->addRow(_('Database type'),
-			new CComboBox('type', $DB['TYPE'], null, CFrontendSetup::getSupportedDatabases())
+		$table->addRow(new CLabel(_('Database type'), 'label-type'),
+			(new CSelect('type'))
+				->setId('type')
+				->setFocusableElementId('label-type')
+				->setValue($DB['TYPE'])
+				->addOptions(CSelect::createOptionsFromArray(CFrontendSetup::getSupportedDatabases()))
 		);
 
 		$table->addRow(_('Database host'),
@@ -297,7 +306,6 @@ class CSetupWizard extends CForm {
 
 		$table->addRow(_('Database port'), [
 			(new CNumericBox('port', $this->getConfig('DB_PORT', '0'), 5, false, false, false))
-				->removeAttribute('style')
 				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
 			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 			(new CSpan(_('0 - use default port')))->addClass(ZBX_STYLE_GREY)
@@ -408,8 +416,9 @@ class CSetupWizard extends CForm {
 		);
 
 		if ($this->STEP_FAILED) {
-			$message_box = makeMessageBox(false, CMessageHelper::getMessages(), _('Cannot connect to the database.'),
-				false, true);
+			$message_box = makeMessageBox(ZBX_STYLE_MSG_BAD, CMessageHelper::getMessages(),
+				_('Cannot connect to the database.'), false, true
+			);
 		}
 		else {
 			$message_box = null;
@@ -435,7 +444,6 @@ class CSetupWizard extends CForm {
 
 		$table->addRow(_('Port'),
 			(new CNumericBox('zbx_server_port', $this->getConfig('ZBX_SERVER_PORT', '10051'), 5, false, false, false))
-				->removeAttribute('style')
 				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 		);
 
@@ -454,16 +462,23 @@ class CSetupWizard extends CForm {
 	}
 
 	private function stage4(): array {
-		$timezones = DateTimeZone::listIdentifiers();
+		$timezones[ZBX_DEFAULT_TIMEZONE] = CDateTimeZoneHelper::getSystemDateTimeZone();
+		$timezones += (new CDateTimeZoneHelper())->getAllDateTimeZones();
 
 		$table = (new CFormList())
-			->addRow(_('Default time zone'),
-				new CComboBox('default_timezone', $this->getConfig('default_timezone'), null,
-					[ZBX_DEFAULT_TIMEZONE => _('System')] + array_combine($timezones, $timezones)
-				)
+			->addRow(new CLabel(_('Default time zone'), 'label-default-timezone'),
+				(new CSelect('default_timezone'))
+					->setValue($this->getConfig('default_timezone', ZBX_DEFAULT_TIMEZONE))
+					->addOptions(CSelect::createOptionsFromArray($timezones))
+					->setFocusableElementId('label-default-timezone')
+					->setAttribute('autofocus', 'autofocus')
 			)
-			->addRow(_('Default theme'),
-				new CComboBox('default_theme', $this->getConfig('default_theme'), 'submit()', APP::getThemes())
+			->addRow(new CLabel(_('Default theme'), 'label-default-theme'),
+				(new CSelect('default_theme'))
+					->setId('default-theme')
+					->setFocusableElementId('label-default-theme')
+					->setValue($this->getConfig('default_theme'))
+					->addOptions(CSelect::createOptionsFromArray(APP::getThemes()))
 			);
 
 		return [
@@ -628,12 +643,23 @@ class CSetupWizard extends CForm {
 
 		$error = false;
 
-		// Create session secret key.
-		if (!$this->dbConnect($db_user, $db_pass) || !CEncryptHelper::updateKey(CEncryptHelper::generateKey())) {
+		/*
+		 * Create session secret key for first installation. If installation already exists, don't make a new key
+		 * because that will terminate the existing session.
+		 */
+		$db_connect = $this->dbConnect($db_user, $db_pass);
+		$is_superadmin = (CWebUser::$data && CWebUser::getType() == USER_TYPE_SUPER_ADMIN);
+		$session_key_update_failed = ($db_connect && !$is_superadmin)
+			? !CEncryptHelper::updateKey(CEncryptHelper::generateKey())
+			: false;
+
+		if (!$db_connect || $session_key_update_failed) {
 			$this->STEP_FAILED = true;
 			$this->setConfig('step', 2);
+
 			return $this->stage2();
 		}
+
 		$this->dbClose();
 
 		if (!$config->save()) {
@@ -649,7 +675,9 @@ class CSetupWizard extends CForm {
 
 			$this->setConfig('ZBX_CONFIG_FILE_CORRECT', false);
 
-			$message_box = makeMessageBox(false, $messages, _('Cannot create the configuration file.'), false, true);
+			$message_box = makeMessageBox(ZBX_STYLE_MSG_BAD, $messages, _('Cannot create the configuration file.'),
+				false, true
+			);
 			$message = [
 				new CTag('p', true, _('Alternatively, you can install it manually:')),
 				new CTag('ol', true, [
