@@ -1510,7 +1510,9 @@ function access_deny($mode = ACCESS_DENY_OBJECT) {
 			$data = [
 				'header' => _('Access denied'),
 				'messages' => [
-					_s('You are logged in as "%1$s".', CWebUser::$data['alias']).' '._('You have no permissions to access this page.'),
+					_s('You are logged in as "%1$s".',
+						CWebUser::$data['username']).' '._('You have no permissions to access this page.'
+					),
 					_('If you think this message is wrong, please consult your administrators about getting the necessary permissions.')
 				],
 				'buttons' => []
@@ -1575,8 +1577,21 @@ function detect_page_type($default = PAGE_TYPE_HTML) {
 	return $default;
 }
 
-function makeMessageBox($good, array $messages, $title = null, $show_close_box = true, $show_details = false) {
-	$class = $good ? ZBX_STYLE_MSG_GOOD : ZBX_STYLE_MSG_BAD;
+/**
+ * Create a message box.
+ *
+ * @param string      $class                  CSS class of the message box. Possible values:
+ *                                            ZBX_STYLE_MSG_GOOD, ZBX_STYLE_MSG_BAD, ZBX_STYLE_MSG_WARNING.
+ * @param array       $messages               An array of messages.
+ * @param string      $messages[]['message']  Message text.
+ * @param string|null $title                  (optional) Message box title.
+ * @param bool        $show_close_box         (optional) Show or hide close button in error message box.
+ * @param bool        $show_details           (optional) Show or hide message details.
+ *
+ * @return CTag
+ */
+function makeMessageBox(string $class, array $messages, string $title = null, bool $show_close_box = true,
+		bool $show_details = false): CTag {
 	$msg_details = null;
 	$link_details = null;
 
@@ -1622,11 +1637,17 @@ function makeMessageBox($good, array $messages, $title = null, $show_close_box =
 		$title = new CSpan($title);
 	}
 
+	$aria_labels = [
+		ZBX_STYLE_MSG_GOOD => _('Success message'),
+		ZBX_STYLE_MSG_BAD => _('Error message'),
+		ZBX_STYLE_MSG_WARNING => _('Warning message')
+	];
+
 	// Details link should be in front of title.
 	$msg_box = (new CTag('output', true, [$link_details, $title, $msg_details]))
 		->addClass($class)
 		->setAttribute('role', 'contentinfo')
-		->setAttribute('aria-label', $good ? _('Success message') : _('Error message'));
+		->setAttribute('aria-label', $aria_labels[$class]);
 
 	if ($show_close_box) {
 		$msg_box->addItem((new CSimpleButton())
@@ -1647,7 +1668,7 @@ function filter_messages(): array {
 	if (!CSettingsHelper::getGlobal(CSettingsHelper::SHOW_TECHNICAL_ERRORS)
 			&& CWebUser::getType() != USER_TYPE_SUPER_ADMIN && !CWebUser::getDebugMode()) {
 		$messages = CMessageHelper::getMessages();
-		CMessageHelper::clear();
+		CMessageHelper::clear(false);
 
 		$generic_exists = false;
 		foreach ($messages as $message) {
@@ -1659,7 +1680,7 @@ function filter_messages(): array {
 				}
 			}
 			else {
-				CMessageHelper::addSuccess($message['message']);
+				CMessageHelper::addMessage($message);
 			}
 		}
 	}
@@ -1668,7 +1689,7 @@ function filter_messages(): array {
 }
 
 /**
- * Returns the message box when messages are present; null otherwise
+ * Returns a message box if there are messages. Otherwise, null.
  *
  * @param  bool    $good            Parameter passed to makeMessageBox to specify message box style.
  * @param  string  $title           Message box title.
@@ -1680,22 +1701,49 @@ function getMessages(bool $good = false, string $title = null, bool $show_close_
 	$messages = get_and_clear_messages();
 
 	$message_box = ($title || $messages)
-		? makeMessageBox($good, $messages, $title, $show_close_box)
+		? makeMessageBox($good ? ZBX_STYLE_MSG_GOOD : ZBX_STYLE_MSG_BAD, $messages, $title, $show_close_box, !$good)
 		: null;
 
 	return $message_box;
 }
 
-function show_messages($good = false, $okmsg = null, $errmsg = null) {
+function show_messages($good = null, $okmsg = null, $errmsg = null) {
 	global $page, $ZBX_MESSAGES_PREPARED;
 
 	if (defined('ZBX_API_REQUEST')) {
 		return null;
 	}
 
-	$title = $good ? $okmsg : $errmsg;
 	$messages = get_and_clear_messages();
 
+	if ($good === null) {
+		$has_errors = false;
+		$has_warnings = false;
+
+		foreach ($messages as $message) {
+			$has_errors = ($has_errors || ($message['type'] === 'error'));
+			$has_warnings = ($has_warnings || ($message['type'] === 'warning'));
+		}
+
+		if ($has_errors) {
+			$class = ZBX_STYLE_MSG_BAD;
+			$good = false;
+		}
+		elseif ($has_warnings) {
+			$class = ZBX_STYLE_MSG_WARNING;
+			$good = true;
+		}
+		else {
+			$class = ZBX_STYLE_MSG_GOOD;
+			$good = true;
+		}
+	}
+	else {
+		$class = $good ? ZBX_STYLE_MSG_GOOD : ZBX_STYLE_MSG_BAD;
+	}
+
+	$title = $good ? $okmsg : $errmsg;
+
 	if ($title === null && !$messages) {
 		return;
 	}
@@ -1716,7 +1764,7 @@ function show_messages($good = false, $okmsg = null, $errmsg = null) {
 			foreach ($messages as $message) {
 				$image_messages[] = [
 					'text' => $message['message'],
-					'color' => $message['type'] == 'error'
+					'color' => ($message['type'] === 'error')
 						? ['R' => 255, 'G' => 55, 'B' => 55]
 						: ['R' => 155, 'G' => 155, 'B' => 55]
 				];
@@ -1768,11 +1816,11 @@ function show_messages($good = false, $okmsg = null, $errmsg = null) {
 
 			// Prepare messages for inclusion within the layout engine.
 			$ZBX_MESSAGES_PREPARED[] = [
-				'is_good' => $good,
+				'class' => $class,
 				'messages' => $messages,
 				'title' => $title,
 				'show_close_box' => true,
-				'show_details' => !$good
+				'show_details' => ($class === ZBX_STYLE_MSG_BAD)
 			];
 
 			break;
@@ -1804,7 +1852,11 @@ function get_prepared_messages(array $options = []): ?string {
 	// Process messages of the current request.
 
 	if ($options['with_current_messages']) {
-		show_messages();
+		show_messages(
+			null,
+			CMessageHelper::getTitle(),
+			CMessageHelper::getTitle()
+		);
 
 		$messages_current = $ZBX_MESSAGES_PREPARED;
 		$restore_messages = [];
@@ -1833,7 +1885,11 @@ function get_prepared_messages(array $options = []): ?string {
 			$failed_attempts
 		));
 
-		show_messages();
+		show_messages(
+			false, // Failed login can be only error message.
+			CMessageHelper::getTitle(),
+			CMessageHelper::getTitle()
+		);
 
 		CProfile::update('web.login.attempt.failed', 0, PROFILE_TYPE_INT);
 	}
@@ -1860,7 +1916,7 @@ function get_prepared_messages(array $options = []): ?string {
 
 	$html = '';
 	foreach (array_merge($messages_authentication, $messages_session, $messages_current) as $box) {
-		$html .= makeMessageBox($box['is_good'], $box['messages'], $box['title'], $box['show_close_box'],
+		$html .= makeMessageBox($box['class'], $box['messages'], $box['title'], $box['show_close_box'],
 			$box['show_details']
 		)->toString();
 	}
@@ -1890,6 +1946,19 @@ function info($msgs): void {
 	}
 }
 
+/**
+ * Add warning messages to the global message array.
+ *
+ * @param array|string $messages
+ */
+function warning($messages): void {
+	zbx_value2array($messages);
+
+	foreach ($messages as $message) {
+		CMessageHelper::addWarning($message);
+	}
+}
+
 /*
  * Add an error to global message array.
  *
@@ -2538,3 +2607,55 @@ function relativeDateToText($from, $to) {
 
 	return $from.' – '.$to;
 }
+
+/**
+ * Get human readable time period.
+ *
+ * @param int $seconds
+ *
+ * @return string
+ */
+function secondsToPeriod(int $seconds): string {
+	$hours = floor($seconds / 3600);
+	$seconds -= $hours * 3600;
+
+	$minutes = floor($seconds / 60);
+	$seconds -= $minutes * 60;
+
+	$period = ($hours > 0) ? _n('%1$s hour', '%1$s hours', $hours) : '';
+
+	if ($minutes > 0) {
+		if ($period !== '') {
+			$period .= ', ';
+		}
+		$period .= _n('%1$s minute', '%1$s minutes', $minutes);
+	}
+
+	if ($seconds > 0 || $period === '') {
+		if ($period !== '') {
+			$period .= ', ';
+		}
+		$period .= _n('%1$s second', '%1$s seconds', $seconds);
+	}
+
+	return $period;
+}
+
+/**
+ * Generates UUID version 4.
+ *
+ * @param string $seed   String to be hashed as md5 and used as UUID body.
+ *
+ * @return string
+ */
+function generateUuidV4($seed = '') {
+	$data = ($seed === '') ? random_bytes(16) : hex2bin(md5($seed));
+
+	// Set head of 7th byte to 0100 (0100xxxx)
+	$data[6] = chr(ord($data[6]) & 0x0f | 0x40);
+
+	// Set head of 9th byte to 10 (10xxxxxx)
+	$data[8] = chr(ord($data[8]) & 0x3f | 0x80);
+
+	return bin2hex($data);
+}
